# Code > Ref > Symbol 계층 구조는 어디서나 공통
# description 과 같은 파라미터에 필요한 추가 정보는 openapi schema의 규칙을 따라야 함
# 참조 https://swagger.io/docs/specification/data-models/keywords/
# https://www.learnjsonschema.com/2020-12/

def get_figure_server_url():
    return 'https://moa.rpm.kr-st-midasit.com/design-guide/figures'

# List of operators
binary_operators = [r'\\pm', r'\\cdot', r'\\times', r'\\div', r'\\mod', r'\\land', r'\\lor', r'\\cup', r'\\cap', r'\\oplus']
relation_operators = [r'=', r'\\neq', r'\\leq', r'\\geq', r'<', r'>', r'\\approx', r'\\sim', r'\\equiv', r'\\subset', r'\\supset']
function_operators = [r'\\sin', r'\\cos', r'\\log', r'\\ln' r'\\lim', r'\\int', r'\\sum', r'\\left', r'\\right']

def find_by_id(comp_id):
    # id 기준으로 node 탐색
    for comp in component_list:
        if comp['id'] == comp_id:
            return comp
    return None

def find_by_latex_symbol(target_latex_symbol):
    # latex_symbol 기준으로 node 탐색
    # NOTE : 중복된 latex_symbol이 있을 수 있음
    for component in component_list:
        if component['latex_symbol'] == target_latex_symbol:
            return component
    return None

def find_comp(latex_symbol, standard, reference):
    for component in component_list:
        if component['latex_symbol'] == latex_symbol and component['standard'] == standard and component['reference'] == reference:
            return component
    return None

def get_table_by_component(comp):
    if 'table' in comp:
        return next(dt for dt in data_table if dt['id'] == comp['id'])
    return None

def get_table_enum_by_component(comp):
    table = get_table_by_component(comp)
    if table and 'enum' in table:
        return table['enum']
    return None

def get_table_criteria_by_component(comp):
    table = get_table_by_component(comp)
    if table and 'criteria' in table:
        return table['criteria']
    return None

def get_table_data_by_component(comp):
    table = get_table_by_component(comp)
    if table and 'data' in table:
        return table['data']
    return None

def convert_enum_table_to_detail(comp):
    enum_table = get_table_enum_by_component(comp)
    if enum_table is None or 'description' not in enum_table[0]:
        return None
        
    detail_table = [[f"{comp['latex_symbol']}", 'Description']]
    for et in enum_table:
        row = []
        row.append(f"{et['label']}")
        if 'description' in et:
            row.append(f"{et['description']}")
        detail_table.append(row)
    return detail_table

component_list = [
    # [EN1991-1-5] Eurocode 1: Actions on structures — Part 1-5: General actions — Thermal actions
    {
        'id': "G1_COMP_1",
        'standard': 'EN1991-1-5',
        'reference': ['6.1.1(1)'],
        'title': 'Deck type selection',
        'description': r"This section categorizes bridge decks into different types based on their material and structural design. The classification includes steel decks, composite decks, and concrete decks, each with distinct subtypes like steel box girders, concrete slabs, and concrete box girders.",
        'figure_file': 'detail_g1_comp_11.png',
        'latex_symbol': r'seldeck',
        'type': 'string',
        'unit': '',
        'notation': 'text',
        'default': 'steel box girder',
        'table': 'dropdown',
    },
    {
        'id': "G1_COMP_2",
        'standard': 'EN1991-1-5',
        'reference': ['6.1.1(1)'],
        'title': 'Bridge deck types',
        'description': r"This section categorizes bridge decks into different types based on their material and structural design. The classification includes steel decks, composite decks, and concrete decks, each with distinct subtypes like steel box girders, concrete slabs, and concrete box girders.",
        'latex_symbol': r'decktype',
        'type': 'string',
        'unit': '',
        'notation': 'text',
        'required': ['G1_COMP_1'],
        'default': 'Type1',
        'table': 'text',
    },
    {
        'id': "G1_COMP_3",
        'standard': 'EN1991-1-5',
        'reference': ['AnnexA.1'],
        'title': 'The initial bridge temperature',
        'description': r"The initial bridge temperature should be taken as the temperature of the structure at the time of restraint or completion. If unpredictable, the average temperature during construction should be used. The initial temperature may also be specified in the National Annex.",
        'latex_symbol': r'T_{0}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 1,
        'limits': {
            'in_min': 0,
        },
        'default' : 10.0,
        'use_std': True,
    },
    {
        'id': 'G1_COMP_4',
        'standard': 'EN1991-1-5',
        'reference': ['1.5.3', '1.5.4'],
        'title': 'Return period',
        'description': r"The return period refers to the number of years after which a specific event, like extreme temperatures, is statistically expected to occur. In Eurocode, the default return period is set to 50 years, but this can be adjusted depending on project requirements or specified in the Annex A.",
        'latex_symbol': r'p_{years}',
        'type': 'number',
        'unit': 'years',
        'notation': 'standard',
        'decimal': 0,
        'limits': {
            'ex_min': 0,
        },
        'default': 50.0,
        'use_std': True,
    },
    {
        'id': "G1_COMP_5",
        'standard': 'EN1991-1-5',
        'reference': ['1.6'],
        'title': 'Annual exceedance probability',
        'description': r"The Annual Exceedance Probability is the probability that a specific event, such as an extreme temperature, will be exceeded in a given year. It is calculated as the inverse of the return period. For example, if the return period is 50 years, the annual exceedance probability is 1/50, or 0.02 (2%). This value helps assess the likelihood of rare events occurring within any given year and is crucial for determining the appropriate safety measures in structural design.",
        'latex_symbol': r'p',
        'latex_equation': r'p = \frac{1}{p_{years}}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G1_COMP_4'],
    },
    {
        'id': 'G1_COMP_6',
        'standard': 'EN1991-1-5',
        'reference': ['6.1.3.2(1)P'],
        'title': 'Minimum shade air temperature (exceedance probability 0.02)',
        'description': r"The minimum shade air temperatures with an annual probability of being exceeded of 0.02 represents the lowest expected air temperature that has a 2% chance of being exceeded in any given year. This corresponds to a return period of 50 years. These values are typically derived from national maps of isotherms and are often specified in the National Annex for structural design.",
        'latex_symbol': r'T_{min}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'default': -17,
        'use_std': False,
    },
    {
        'id': 'G1_COMP_7',
        'standard': 'EN1991-1-5',
        'reference': ['6.1.3.2(1)P'],
        'title': 'Maximum shade air temperature (exceedance probability 0.02)',
        'description': r"The maximum shade air temperatures with an annual probability of being exceeded of 0.02 refers to the highest expected air temperature that has a 2% chance of being exceeded in any given year, equivalent to a 50-year return period. These values are derived from national maps of isotherms and are often specified in the National Annex for use in structural design.",
        'latex_symbol': r'T_{max}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'default': 34,
        'use_std': False,
    },
    {
        'id': 'G1_COMP_8',
        'standard': 'EN1991-1-5',
        'reference': ['AnnexA.2(2)'],
        'title': 'Minimum shade air temperature with exceedance probability p',
        'description': r"The minimum shade air temperature represents the temperature with an annual probability of being exceeded by p, which is different from the standard 0.02 exceedance probability. This value corresponds to a return period of 1/p years, meaning it reflects the minimum air temperature expected to be exceeded once every 1/p years on average, depending on the chosen probability. These values are critical for projects that require specific safety measures based on local climate conditions.",
        'latex_symbol': r'T_{min,p}',
        'latex_equation': r'T_{min,p} = T_{min} \times \left{ k_{3} + k_{4} \times \ln \left[ - \ln \left( 1 - p \right) \right] \right \right}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G1_COMP_6', 'G1_COMP_12', 'G1_COMP_13', 'G1_COMP_5'],
    },
    {
        'id': 'G1_COMP_9',
        'standard': 'EN1991-1-5',
        'reference': ['AnnexA.2(2)'],
        'title': 'Maximum shade air temperature with exceedance probability p',
        'description': r"The maximum shade air temperature represents the temperature with an annual probability of being exceeded by p, which differs from the standard exceedance probability of 0.02. This value is associated with a return period of 1/p years, meaning it reflects the highest air temperature expected to be exceeded once every 1/p years on average. These values are crucial for projects requiring adjustments based on local climate conditions to ensure the structural safety of the design.",
        'latex_symbol': r'T_{max,p}',
        'latex_equation': r'T_{max,p} = T_{max} \times \left{ k_{1} - k_{2} \times \ln \left[ - \ln \left( 1 - p \right) \right] \right}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G1_COMP_7', 'G1_COMP_10', 'G1_COMP_11', 'G1_COMP_5'],
    },
    {
        'id': 'G1_COMP_10',
        'standard': 'EN1991-1-5',
        'reference': ['AnnexA.2(2)'],
        'title': 'Coefficient for maximum shade air temperature',
        'description': r"This coefficient is used for calculating the maximum air temperature with a specified probability (p). It depends on the variation coefficient and can be modified using meteorological data for the mean value and standard deviation.",
        'latex_symbol': r'k_{1}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 0.781,
        'const': True,
    },
    {
        'id': 'G1_COMP_11',
        'standard': 'EN1991-1-5',
        'reference': ['AnnexA.2(2)'],
        'title': 'Scaling factor for maximum shade air temperature',
        'description': r"This coefficient serves as a scaling factor for the maximum air temperature calculation. It is influenced by the variation coefficient and can be adjusted by meteorological data.",
        'latex_symbol': r'k_{2}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 0.056,
        'const': True,
    },
    {
        'id': 'G1_COMP_12',
        'standard': 'EN1991-1-5',
        'reference': ['AnnexA.2(2)'],
        'title': 'Coefficient for minimum shade air temperature',
        'description': r"This coefficient is used to calculate the minimum (negative) air temperature with a specified probability (p). It relates to the variation coefficient for negative temperatures and can be modified using meteorological data.",
        'latex_symbol': r'k_{3}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 0.393,
        'const': True,
    },
    {
        'id': 'G1_COMP_13',
        'standard': 'EN1991-1-5',
        'reference': ['AnnexA.2(2)'],
        'title': 'Scaling factor for minimum shade air temperature',
        'description': r"Similar to k2, this coefficient scales the calculation for minimum (negative) air temperatures. It is based on the variation coefficient for negative temperatures and can be adjusted using meteorological data.",
        'latex_symbol': r'k_{4}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': -0.156,
        'const': True,
    },
    {
        'id': 'G1_COMP_14',
        'standard': 'EN1991-1-5',
        'reference': ['6.1.3.1(Figure6.1)'],
        'title': 'Minimum uniform bridge temperature component',
        'description': r"The minimum uniform bridge temperature component represents the lowest temperature that a bridge structure can uniformly experience. It should be determined based on local climatic conditions and national guidelines. The values are typically derived from a correlation with the minimum shade air temperature. National Annexes may specify exact values, and recommended ranges are provided.",
        'latex_symbol': r'T_{e,min}',
        'latex_equation': r'T_{e,min} = T_{min,p} - 3',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G1_COMP_2', 'G1_COMP_8'],
        'table': 'formula',
    },
    {
        'id': 'G1_COMP_15',
        'standard': 'EN1991-1-5',
        'reference': ['6.1.3.1(Figure6.1)'],
        'title': 'Maximum uniform bridge temperature component',
        'description': r"The maximum uniform bridge temperature component represents the highest temperature that a bridge structure can uniformly experience. Like the minimum uniform temperature, it is determined based on local climate and national guidelines. This component is correlated with the maximum shade air temperature. The National Annex may provide specific values, and appropriate ranges are suggested.",
        'latex_symbol': r'T_{e,max}',
        'latex_equation': r'T_{e,max} = T_{max,p} + 16',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G1_COMP_2', 'G1_COMP_9'],
        'table': 'formula',
    },
    {
        'id': 'G1_COMP_16',
        'standard': 'EN1991-1-5',
        'reference': ['6.1.3.3(3)'],
        'title': 'Maximum contraction range of a uniform bridge temperature',
        'description': r"The maximum contraction range of a uniform bridge temperature component represents the temperature difference between the initial bridge temperature and the minimum uniform bridge temperature. This value reflects how much the bridge may contract as its temperature drops from the initial temperature to the minimum operational temperature.",
        'latex_symbol': r'\Delta T_{N,con}',
        'latex_equation': r'\Delta T_{N,con} = T_{0} - T_{e,min}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G1_COMP_3', 'G1_COMP_14'],
    },
    {
        'id': 'G1_COMP_17',
        'standard': 'EN1991-1-5',
        'reference': ['6.1.3.3(3)'],
        'title': 'Maximum expansion range of a uniform bridge temperature',
        'description': r"The maximum expansion range of a uniform bridge temperature represents the temperature difference between the maximum uniform bridge temperature and the initial bridge temperature. This value indicates how much the bridge may expand as its temperature increases from the initial temperature to the maximum operational temperature.",
        'latex_symbol': r'\Delta T_{N,exp}',
        'latex_equation': r'\Delta T_{N,exp} = T_{e,max} - T_{0}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G1_COMP_15', 'G1_COMP_3'],
    },

    # [EN1992-1-1] Eurocode 2: Design of concrete structures — Part 1-1: General rules and rules for buildings
    {
        'id': 'G2_COMP_1',
        'standard': 'EN1992-1-1',
        'reference': ['5.2(5.1)'],
        'title': 'Geometrical imperfections in bridge',
        'description': r"Inclination caused by geometrical imperfections refers to the initial deviation in the structure's alignment due to fabrication or construction errors. It affects the stability of structural members like girders, piers, and arches, and must be accounted for in design to ensure the bridge's overall stability under load. For bridge-specific details, refer to EN1992-2(5.101).",
        'latex_symbol': r'\theta_{i}',
        'latex_equation': r'\theta_{i} = \theta_{0} \times \alpha_{h}',
        'type': 'number',
        'unit': 'rad',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G2_COMP_2', 'G2_COMP_3'],
    },
    {
        'id': 'G2_COMP_2',
        'standard': 'EN1992-1-1',
        'reference': ['5.2(5)'],
        'title': 'Basic value of initial imperfections in bridge',
        'description': r"The basic value of initial imperfections in bridge structures represents the initial deviation or inclination due to construction or fabrication inaccuracies. This value, often specified in national standards, is typically set as 1/200 and serves as the foundation for calculating the overall imperfections in structural members such as girders, piers, and arches.",
        'latex_symbol': r'\theta_{0}',
        'type': 'number',
        'unit': 'rad',
        'notation': 'standard',
        'decimal': 3,
        'default': 0.005,
        'const': True,
    },
    {
        'id': 'G2_COMP_3',
        'standard': 'EN1992-1-1',
        'reference': ['5.2(5)'],
        'title': 'Reduction factor for height or length in bridge',
        'description': r"The reduction factor for height or length in bridge structures is used to adjust the basic value of initial imperfections. This factor accounts for the length or height of structural elements like girders, piers, or arches, ensuring that imperfections are scaled appropriately based on the size of the structure. For more detailed information, refer to EN1992-2(5.2).",
        'latex_symbol': r'\alpha_{h}',
        'latex_equation': r'\alpha_{h} = \min \left( \frac{2}{\sqrt{l}}, 1 \right)',
        'type': 'number',
        'unit': 'rad',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G2_COMP_4'],
    },
    {
        'id': 'G2_COMP_4',
        'standard': 'EN1992-1-1',
        'reference': ['5.2(5)'],
        'title': 'Length or height of structural members in bridge design',
        'description': r"The length or height of structural members is used to calculate the reduction factor for geometrical imperfections, reflecting the actual size of elements like girders, piers, and arches in bridge design.",
        'latex_symbol': r'l',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 30,
        'use_std': False,
    },
    {
        'id': 'G2_COMP_5',
        'standard': 'EN1992-1-1',
        'reference': ['5.2(5.2)'],
        'title': 'Eccentricity due to geometrical imperfections in isolated members',
        'description': r"In bridge design, eccentricity due to geometrical imperfections in isolated members refers to the deviation of the structural axis, which results in additional bending moments. This provides a simplified way to account for imperfections in structural analysis, particularly in compression elements like columns and girders.",
        'latex_symbol': r'e_{i}',
        'latex_equation': r'e_{i} = \theta_{i} \times \left( \frac{l_{0}}{2} \right)',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G2_COMP_1', 'G2_COMP_8'],
    },
    {
        'id': 'G2_COMP_6',
        'standard': 'EN1992-1-1',
        'reference': ['5.2(7)'],
        'title': 'Transverse force due to imperfections in isolated members',
        'description': r"The transverse force due to imperfections in isolated members refers to the lateral force applied at the position that creates the maximum moment. This force accounts for geometrical imperfections such as initial deflections and is used to ensure accurate buckling analysis in structural design.",
        'latex_symbol': r'H_{i}',
        'latex_equation': r'H_{i} = N \times e_{i} \times \left( \frac{4}{l} \right)',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G6_COMP_2', 'G2_COMP_4', 'G2_COMP_5', 'G2_COMP_7'],
        'table': 'formula',
    },
    {
        'id': 'G2_COMP_7',
        'standard': 'EN1992-1-1',
        'reference': ['5.2(7)'],
        'title': 'Axial load in isolated members',
        'description': r"Axial load in isolated members refers to the vertical or compressive force applied along the axis of a structural element. This load is crucial in analyzing buckling behavior, as it interacts with imperfections such as initial deflection or inclination, affecting the stability of the member.",
        'latex_symbol': r'N',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 650,
        'use_std': False,
    },
    {
        'id': 'G2_COMP_8',
        'standard': 'EN1992-1-1',
        'reference': ['5.2(7)'],
        'title': 'Effective length',
        'description': r"The effective length of isolated members refers to the length used in buckling analysis to represent the portion of a structural member that is free to buckle under load. This length depends on the boundary conditions, such as pinned, fixed, or free ends. It is essential for predicting when and how isolated members, like girders or columns, will buckle. For more detailed information, please refer to Section 5.8.3.2 (Figure 5.7).",
        'latex_symbol': r'l_{0}',
        'latex_equation': r'l_{0} = l',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G6_COMP_2', 'G2_COMP_4'],
        'table': 'formula',
    },
    
    # [EN1991-1-4] Eurocode 1: Actions on structures — Part 1-4: General actions — Wind actions
    {
        'id': 'G3_COMP_1',
        'standard': 'EN1991-1-4',
        'reference': ['4.2(4.1)'],
        'title': 'Basic wind velocity',
        'description': r"The basic wind velocity refers to the mean wind speed calculated over a 10-minute period at a height of 10 meters above the ground in terrain with minimal vegetation and few obstacles. It takes into account factors like wind direction and the time of year and is adjusted using directional and seasonal factors.",
        'latex_symbol': r'v_{b}',
        'latex_equation': r'v_{b} = c_{dir} \times c_{season} \times v_{b,0} \times c_{prob}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_3', 'G3_COMP_4', 'G3_COMP_2', 'G3_COMP_5'],
    },
    {
        'id': 'G3_COMP_2',
        'standard': 'EN1991-1-4',
        'reference': ['4.2(1)P'],
        'title': 'Fundamental wind velocity',
        'description': r"The fundamental wind velocity is the baseline 10-minute mean wind speed measured at a height of 10 meters above open terrain with minimal vegetation and few obstacles. It is the starting point for calculating the basic wind velocity, before applying any adjustments for wind direction or season. The value is typically provided in national standards.",
        'latex_symbol': r'v_{b,0}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 27,
        'use_std': False,
    },
    {
        'id': 'G3_COMP_3',
        'standard': 'EN1991-1-4',
        'reference': ['4.2(2)P'],
        'title': 'Wind directional adjustment factor',
        'description': r"The directional factor is a coefficient that adjusts the basic wind velocity based on the wind direction. It accounts for the variability in wind speeds depending on which direction the wind is coming from. The value of this factor may vary depending on national standards, but a common recommended value is 1.0.",
        'latex_symbol': r'c_{dir}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G3_COMP_4',
        'standard': 'EN1991-1-4',
        'reference': ['4.2(2)P'],
        'title': 'Seasonal adjustment factor',
        'description': r"The seasonal adjustment factor accounts for changes in wind speed due to seasonal variations. It modifies the basic wind velocity to reflect different wind conditions throughout the year. The recommended value is often 1.0, indicating no adjustment for seasons.",
        'latex_symbol': r'c_{season}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G3_COMP_5',
        'standard': 'EN1991-1-4',
        'reference': ['4.2(4.2)'],
        'title': 'Wind probability adjustment factor',
        'description': r"The wind probability adjustment factor modifies the basic wind velocity based on the likelihood of extreme wind speeds occurring within a given year. It adjusts the wind speed to account for the probability of these events, using specific statistical parameters related to the distribution of extreme values. These parameters are typically provided in national standards.",
        'latex_symbol': r'c_{prob}',
        'latex_equation': r'c_{prob} = \left( \frac{ \left(1 - K \times \ln \left( -\ln \left(1 - p \right) \right) \right) }{ 1 - K \times \ln \left(-\ln \left( 0.98 \right) \right) } \right)^{n}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_8', 'G3_COMP_9', 'G3_COMP_7'],
    },
    {
        'id': 'G3_COMP_6',
        'standard': 'EN1991-1-4',
        'reference': ['3.4(1)'],
        'title': 'Mean return period',
        'description': r"The mean return period is the average time interval over which a specific wind event (like a wind speed) is expected to be exceeded once. A mean return period of 50 years indicates that the wind speed is likely to be exceeded once every 50 years on average.",
        'latex_symbol': r'p_{years}',
        'type': 'number',
        'unit': 'years',
        'notation': 'standard',
        'decimal': 0,
        'limits': {
            'ex_min': 0,
        },
        'default': 50,
        'use_std': True,
    },
    {
        'id': 'G3_COMP_7',
        'standard': 'EN1991-1-4',
        'reference': ['3.4(1)'],
        'title': 'Annual exceedance probability',
        'description': r"This refers to the likelihood that a specific wind speed will be exceeded in any given year. An annual probability of exceedence of 0.02 means there is a 2% chance that the wind speed will exceed this value in one year.",
        'latex_symbol': r'p',
        'latex_equation': r'p = \frac{1}{(p_{years})}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_6'],
    },
    {
        'id': 'G3_COMP_8',
        'standard': 'EN1991-1-4',
        'reference': ['4.2(2)P'],
        'title': 'Wind speed shape parameter for extreme events',
        'description': r"The wind speed shape parameter for extreme events is a statistical coefficient that characterizes the distribution of extreme wind speeds. It influences the adjustment of the probability factor, reflecting the variability and likelihood of extreme wind conditions. This parameter is typically determined based on the variation in wind speed data and is provided in national standards.",
        'latex_symbol': r'K',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'default': 0.2,
        'const': True,
    },
    {
        'id': 'G3_COMP_9',
        'standard': 'EN1991-1-4',
        'reference': ['4.2(2)P'],
        'title': 'Exponent for wind probability adjustment',
        'description': r"The exponent for wind probability adjustment is a factor used to modify the probability factor in wind speed calculations. It determines the degree of influence that the probability of extreme wind events has on the adjusted wind velocity.",
        'latex_symbol': r'n',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'default': 0.5,
        'const': True,
    },
    {
        'id': 'G3_COMP_10',
        'standard': 'EN1991-1-4',
        'reference': ['5.1(Table5.1)'],
        'title': 'Reference height above ground',
        'description': r"The reference height is the maximum height above ground for the section being considered. This height is used in wind load calculations to determine the peak wind velocity and other wind-related forces acting on the structure.",
        'latex_symbol': r'z_{e}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 6,
        'use_std': False,
    },
    {
        'id': 'G3_COMP_11',
        'standard': 'EN1991-1-4',
        'reference': ['4.3.1(1)'],
        'title': 'Height above ground level',
        'description': r"The height above ground level refers to the vertical distance from the ground where wind speed measurements or calculations are taken. It is a key parameter in determining wind velocity, as wind speed varies with height due to changes in terrain roughness and other factors.",
        'latex_symbol': r'z',
        'latex_equation': r'z = \min \left(z_{e}, z_{max} \right)',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_10', 'G3_COMP_20'],
    },
    {
        'id': 'G3_COMP_12',
        'standard': 'EN1991-1-4',
        'reference': ['4.3.1(4.3)'],
        'title': 'Mean wind velocity at height',
        'description': r"The mean wind velocity at height refers to the average wind speed at a specific height above the ground. This value is influenced by factors such as the roughness of the terrain, the topography (orography), and the basic wind velocity. It is calculated using the roughness and orography adjustment factors, which modify the wind speed to reflect the effects of the surrounding environment at different heights.",
        'latex_symbol': r'v_{m}(z)',
        'latex_equation': r'v_{m}(z) = c_{r}(z) \times c_{o}(z) \times v_{b}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_14', 'G3_COMP_13', 'G3_COMP_1'],
    },
    {
        'id': 'G3_COMP_13',
        'standard': 'EN1991-1-4',
        'reference': ['4.3.1(1)'],
        'title': 'Orography adjustment factor for mean wind speed',
        'description': r"The orography adjustment factor for mean wind speed takes into account the effects of terrain features such as hills or valleys on wind speed at various heights. It adjusts the mean wind speed based on these topographical variations. Typically, the value is set to 1.0 unless specified otherwise in national standards, indicating no adjustment for orography.",
        'latex_symbol': r'c_{o}(z)',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G3_COMP_14',
        'standard': 'EN1991-1-4',
        'reference': ['4.3.2(4.4)'],
        'title': 'Terrain roughness factor for mean wind velocity',
        'description': r"The terrain roughness factor adjusts the mean wind velocity to account for changes in wind speed caused by the roughness of the terrain and the height above the ground. It is determined using a logarithmic formula and varies based on the roughness of the terrain and specific height.",
        'latex_symbol': r'c_{r}(z)',
        'latex_equation': r'c_{r(z_{min})} = k_{r} \times \ln \left(\frac {z_{min}}{z_{0}} \right)',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_11', 'G3_COMP_17', 'G3_COMP_16', 'G3_COMP_18'],
        'table': 'formula',
    },
    {
        'id': "G3_COMP_15",
        'standard': 'EN1991-1-4',
        'reference': ['4.3.2(Table4.1)'],
        'title': 'Terrain category selection',
        'description': r"This section allows the user to choose the appropriate terrain category based on the characteristics of the surrounding area. Terrain categories are used to define the roughness of the ground, which affects wind speed calculations. Options typically range from open terrain with low vegetation to urban environments with numerous obstacles.",
        'figure_file': 'detail_g3_comp_15.png',
        'latex_symbol': r'terrain',
        'type': 'string',
        'unit': '',
        'notation': 'text',
        'default': r'[0] Sea or coastal area exposed to the open sea',
        'table': 'dropdown',
    },
    {
        'id': "G3_COMP_16",
        'standard': 'EN1991-1-4',
        'reference': ['4.3.2(Table4.1)'],
        'title': 'Roughness length for terrain',
        'description': r"The roughness length is a parameter that represents the height above ground level where the wind speed theoretically becomes zero due to surface roughness. It is used in the calculation of the terrain roughness factor and depends on the type of terrain. The value of the roughness length varies according to different terrain categories.",
        'latex_symbol': r'z_{0}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_15'],
        'default': 0.003,
        'table' : 'formula',
    },
    {
        'id': "G3_COMP_17",
        'standard': 'EN1991-1-4',
        'reference': ['4.3.2(Table4.1)'],
        'title': 'Minimum height for roughness factor calculation',
        'description': r"The minimum height is the lowest height above the ground at which the roughness factor is applicable. Below this height, the roughness factor remains constant. The value of the minimum height is determined by the terrain category and is used in the calculation of wind speed adjustments.",
        'latex_symbol': r'z_{min}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 1,
        'required': ['G3_COMP_15'],
        'default': 1,
        'table' : 'formula',
    },
    {
        'id': 'G3_COMP_18',
        'standard': 'EN1991-1-4',
        'reference': ['4.3.2(4.5)'],
        'title': 'Terrain factor for roughness calculation',
        'description': r"The terrain factor is a coefficient used in the calculation of the roughness factor, reflecting the influence of the terrain's roughness length. It depends on the ratio between the roughness length of the terrain and the standard roughness length for a reference terrain (terrain category II). This factor helps adjust wind speed based on different terrain conditions.",
        'latex_symbol': r'k_{r}',
        'latex_equation': r'k_{r} = 0.19 \times \left( \frac{z_{0}}{z_{0,II}} \right) ^ {0.07}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_16', 'G3_COMP_19'],
    },
    {
        'id': 'G3_COMP_19',
        'standard': 'EN1991-1-4',
        'reference': ['4.3.2(Table4.1)'],
        'title': 'Reference roughness length for terrain category II',
        'description': r"The reference roughness length for terrain category II is a standard value used as a baseline for calculating the terrain factor in wind speed assessments. It represents the roughness length for open terrain with low vegetation, typically set at 0.05 meters. This value is used to compare other terrain roughness lengths and adjust the wind speed accordingly.",
        'latex_symbol': r'z_{0,II}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 2,
        'default': 0.05,
        'const': True,
    },
    {
        'id': 'G3_COMP_20',
        'standard': 'EN1991-1-4',
        'reference': ['4.3.2(1)'],
        'title': 'Maximum height for roughness factor calculation',
        'description': r"The maximum height is the highest altitude above ground where the roughness factor is considered in wind speed calculations. Above this height, the roughness factor does not apply. For most cases, the maximum height is set at 200 meters, unless otherwise specified.",
        'latex_symbol': r'z_{max}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 1,
        'default': 200,
        'const': True,
    },
    {
        'id': 'G3_COMP_21',
        'standard': 'EN1991-1-4',
        'reference': ['4.5(4.8)'],
        'title': 'Standard deviation of wind turbulence',
        'description': r"The peak velocity pressure at height accounts for both the mean wind velocity and short-term wind fluctuations at a specific height above the ground. It is calculated using the air density, mean wind velocity, and an exposure factor, which reflects the terrain and other local conditions. The peak velocity pressure helps in determining the wind loads on structures.",
        'latex_symbol': r'\sigma_{v}',
        'latex_equation': r'\sigma_{v} = k_{r} \times v_{b} \times k_{l}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_18', 'G3_COMP_1', 'G3_COMP_22'],
    },
    {
        'id': 'G3_COMP_22',
        'standard': 'EN1991-1-4',
        'reference': ['4.4(1)'],
        'title': 'Turbulence factor',
        'description': r"The turbulence factor is a coefficient used in the calculation of wind turbulence intensity. It accounts for the variability of turbulence in different wind conditions and terrains. The value of the turbulence factor may be specified in national standards, with a commonly recommended value being 1.0. This factor adjusts the standard deviation of wind turbulence in relation to the basic wind velocity.",
        'latex_symbol': r'k_{l}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G3_COMP_23',
        'standard': 'EN1991-1-4',
        'reference': ['4.4(4.7)'],
        'title': 'Turbulence intensity at height',
        'description': r"Turbulence intensity at height is the ratio of the standard deviation of wind turbulence to the mean wind velocity at a specific height. It measures the level of wind speed fluctuations due to turbulence. The value of turbulence intensity depends on the terrain roughness and height above ground. It is calculated using a formula that incorporates factors such as the roughness length and orography.",
        'latex_symbol': r'I_{v}(z)',
        'latex_equation': r'I_{v}(z) = \frac{k_{l}}{c_{o}(z) \times \ln \left( \frac{z_{min}}{z_{0}} \right)}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_22', 'G3_COMP_13', 'G3_COMP_11', 'G3_COMP_17', 'G3_COMP_16'],
        'table': 'formula',
    },
    {
        'id': 'G3_COMP_24',
        'standard': 'EN1991-1-4',
        'reference': ['4.5(4.8)'],
        'title': 'Peak velocity pressure at height',
        'description': r"The peak velocity pressure at height accounts for both the mean wind velocity and short-term wind fluctuations at a specific height above the ground. It is calculated using the air density, mean wind velocity, and an exposure factor, which reflects the terrain and other local conditions. The peak velocity pressure helps in determining the wind loads on structures.",
        'latex_symbol': r'q_{p}(z)',
        'latex_equation': r'q_{p}(z) = \left[ 1 + 7 \times I_{v}(z)  \right] \cdot \frac{1}{2} \times \rho \times v_{m}(z)^{2} \times 10^{-3}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_23', 'G3_COMP_25', 'G3_COMP_12'],
    },
    {
        'id': 'G3_COMP_25',
        'standard': 'EN1991-1-4',
        'reference': ['4.5(1)'],
        'title': 'Air density during wind storms',
        'description': r"Air density during wind storms is a crucial factor in wind load calculations, as it depends on altitude, temperature, and barometric pressure expected in the region during extreme wind events. This parameter affects the calculation of peak velocity pressure and can vary with changing atmospheric conditions.",
        'latex_symbol': r'\rho',
        'type': 'number',
        'unit': 'kg/m^3',
        'notation': 'standard',
        'decimal': 3,
        'default': 1.25,
        'const': True,
    },
    {
        'id': 'G3_COMP_26',
        'standard': 'EN1991-1-4',
        'reference': ['4.5(4.9)'],
        'title': 'Exposure factor at heigh',
        'description': r"The exposure factor at height adjusts the peak velocity pressure based on the height above ground and the surrounding terrain conditions. It reflects the influence of terrain roughness and exposure to wind, varying with the type of terrain and height. This factor helps modify the peak wind pressure to better represent real-world conditions at different elevations.",
        'latex_symbol': r'c_{e}(z)',
        'latex_equation': r'c_{e}(z) = \frac{q_{p}(z)}{q_{b}}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_24', 'G3_COMP_27'],
    },
    {
        'id': 'G3_COMP_27',
        'standard': 'EN1991-1-4',
        'reference': ['4.5(4.10)'],
        'title': 'Basic velocity pressure',
        'description': r"The basic velocity pressure represents the wind pressure at a given location based on the basic wind velocity. It is calculated using air density and the square of the basic wind velocity. This value serves as a reference for further adjustments when calculating the peak velocity pressure at different heights or under different terrain conditions.",
        'latex_symbol': r'q_{b}',
        'latex_equation': r'q_{b} = \left( \frac{1}{2} \times \rho \times v_{b}^{2} \right) \times 10^{-3}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_25', 'G3_COMP_1'],
    },

    # [EN1991-1-1] Eurocode 1: Actions on structures — Part 1-1: General actions — Densities, self-weight, imposed loads for buildings
    {
        'id': 'G4_COMP_1',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA'],
        'title': 'Density Type',
        'description': r"'Density Type' clearly categorizes the different forms of density, such as 'Mass Density' and 'Weight Density.' This title helps distinguish between how density is expressed, whether as mass per unit volume (kg/m³) or weight per unit volume (kN/m³), ensuring clarity in the table.",
        'latex_symbol': r'densitytype',
        'type': 'string',
        'unit': '',
        'notation': 'text',
        'default': "Weight density (kN/m^{3})",
        'table': 'dropdown',
    },
    {
        'id': 'G4_COMP_2',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA'],
        'title': 'Gravity acceleration',
        'description': r"Gravity acceleration is applied to convert weight density to mass density. The value of gravity acceleration follows Eurocode standards. Detailed information can be found in EN 1998-1 Section 1.7.",
        'latex_symbol': r'g',
        'type': 'number',
        'unit': 'm/s^2',
        'notation': 'standard',
        'decimal': 3,
        'default': '9.81',
        'const': True,
    },
    {
        'id': 'G4_COMP_3',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA'],
        'title': 'kN to kg unit converter',
        'description': r"This logic converts between kilonewtons (kN) and kilograms (kg) by applying gravitational acceleration, offering an efficient way to handle force and mass conversions.",
        'latex_symbol': r'kntokg',
        'latex_equation': r'kntokg = 1',
        'type': 'number',
        'unit': 'm/s^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G4_COMP_1', 'G4_COMP_2'],
        'table': 'formula',
    },
    {
        'id': 'G4_COMP_4',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA(Table A.1)'],
        'title': 'Density of concrete material',
        'description': r"This menu provides the density corresponding to the selected type of concrete, allowing users to easily find the appropriate density for normal weight, reinforced, or pre-stressed concrete.",
        'latex_symbol': r'conmater',
        'type': 'object',
        'unit': '',
        'notation': 'standard',
        'decimal': 2,
        'required': ['G4_COMP_3'],
        'table': 'result',
    },
    {
        'id': 'G4_COMP_5',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA(Table A.6)'],
        'title': 'Pavement of road bridges',
        'description': r"This section provides the density values for different types of asphalt used in road bridge pavements, including Guss Asphalt for waterproofing and bridge decks, Mastic Asphalt for dense and smooth surfaces, and Hot Rolled Asphalt (HRA) for durable, wear-resistant applications.",
        'latex_symbol': r'paveroad',
        'type': 'object',
        'unit': '',
        'notation': 'standard',
        'decimal': 2,
        'required': ['G4_COMP_3'],
        'table': 'result',
    },
    {
        'id': 'G4_COMP_6',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA(Table A.6)'],
        'title': 'Pavement of rail bridges',
        'description': r"This section provides the density values for materials used in rail bridge pavements, including the concrete protective layer, normal ballast (e.g., granite, gneiss), and basaltic ballast. These materials are essential for ensuring the durability and stability of the bridge surface.",
        'latex_symbol': r'paverail',
        'type': 'object',
        'unit': '',
        'notation': 'standard',
        'decimal': 2,
        'required': ['G4_COMP_3'],
        'table': 'result',
    },
    {
        'id': 'G4_COMP_7',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA(Table A.6)'],
        'title': 'Infill for bridges',
        'description': r"This section lists the density values for various infill materials used in bridge construction, including ballast, gravel (loose), hardcore, crushed slag, packed stone rubble, and puddle clay. These materials are essential for providing stability, drainage, and support in bridge structures.",
        'latex_symbol': r'infillbridge',
        'type': 'object',
        'unit': 'm/s^2',
        'notation': 'standard',
        'decimal': 2,
        'required': ['G4_COMP_3'],
        'table': 'result',
    },
    {
        'id': 'G4_COMP_8',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA(Table A.6)'],
        'title': 'Structures with ballasted bed',
        'description': r"This section provides the density values for various track components used in structures with ballasted beds, including 2 rails UIC 60, prestressed concrete sleepers with track fastenings, concrete sleepers with metal angle braces, and timber sleepers with track fastenings.",
        'latex_symbol': r'wballa',
        'type': 'object',
        'unit': '',
        'notation': 'standard',
        'decimal': 2,
        'required': ['G4_COMP_3'],
        'table': 'result',
    },
    {
        'id': 'G4_COMP_9',
        'standard': 'EN1991-1-1',
        'reference': ['AnnexA(Table A.6)'],
        'title': 'Structures without ballasted bed',
        'description': r"This section provides the density values for components used in structures without a ballasted bed, including 2 rails UIC 60 with track fastenings, bridge beams, and guard rails.",
        'latex_symbol': r'woballa',
        'type': 'object',
        'unit': '',
        'notation': 'standard',
        'decimal': 2,
        'required': ['G4_COMP_3'],
        'table': 'result',
    },

    # [EN1991-1-4] Eurocode 1: Actions on structures — Part 1-4: General actions — Wind actions
    {
        'id': 'G5_COMP_1',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.2(8.2)'],
        'title': 'Wind force acting in the x-direction on bridges',
        'description': r"The wind force acting in the x-direction on bridges refers to the horizontal wind load exerted perpendicular to the bridge span. This force impacts the width of the deck and is influenced by factors such as basic wind speed, air density, and the shape of the bridges. It is calculated to ensure the structural integrity of the bridges under wind pressure. When wind forces and traffic loads occur simultaneously, adjustments are made to account for both effects to maintain safety.",
        'latex_symbol': r'F_{w}',
        'latex_equation': r'F_{w} = \frac{1}{2} \times \rho \times v_{b}^{2} \times C \times A_{ref,x} \times 10^{-3}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_25', 'G3_COMP_1', 'G5_COMP_2', 'G5_COMP_5'],
    },
    {
        'id': 'G5_COMP_2',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.2(1)'],
        'title': 'Wind load factor acting on bridges',
        'description': r"The wind load factor adjusts the wind pressure on a bridge based on its exposure to wind and aerodynamic shape. It combines the effects of the exposure and shape factors to ensure accurate calculation of wind forces.",
        'latex_symbol': r'C',
        'latex_equation': r'C = c_{e}(z) \times c_{f,x}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_26', 'G5_COMP_3'],
    },
    {
        'id': 'G5_COMP_3',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(8.1)'],
        'title': 'Force coefficient in the x-direction',
        'description': r"The force coefficient in the x-direction represents the wind resistance on a bridge deck from wind acting perpendicular to the span. For standard bridges, it is usually taken as 1.3, or determined from charts. Special cases may apply for steep wind angles or multiple bridge decks.",
        'latex_symbol': r'c_{f,x}',
        'latex_equation': r'c_{f,x} = c_{fx,0}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G5_COMP_4'],
    },
    {
        'id': 'G5_COMP_4',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(1)'],
        'title': 'Base force coefficient in the x-direction',
        'description': r"The base force coefficient in the x-direction represents the wind resistance of a bridge deck without considering free-end flow. It can either be taken as 1.3 for standard bridges, or calculated based on the b/d_{tot} ratio.",
        'latex_symbol': r'c_{fx,0}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.3,
        'const': True,
    },
    {
        'id': 'G5_COMP_5',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(Figure8.3)'],
        'title': 'Reference area in the x-direction',
        'description': r"The reference area in the x-direction is calculated by multiplying the total depth of the bridge deck by the length of the bridge. This area represents the surface exposed to wind forces acting perpendicular to the span and is used in wind load calculations.",
        'latex_symbol': r'A_{ref,x}',
        'latex_equation': r'A_{ref,x} = d_{tot} \times L',
        'type': 'number',
        'unit': 'm^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G5_COMP_10', 'G5_COMP_6'],
    },
    {
        'id': 'G5_COMP_6',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(Figure8.3)'],
        'title': 'Length of the bridge',
        'description': r"The length of the bridge, denoted as L, refers to the horizontal distance measured along the span of the bridge from one end to the other. This dimension is important in wind load calculations, as it helps determine the surface area of the bridge exposed to wind forces, particularly in the x and z directions. The length L is used to calculate the reference areas for wind loads acting on the bridge deck.",
        'latex_symbol': r'L',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 10,
        'use_std': False,
    },
    {
        'id': 'G5_COMP_7',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(Table8.1)'],
        'title': 'Depth of the bridge deck',
        'description': r"The depth of the bridge deck refers to the vertical distance from the top surface of the deck to the bottom of the main supporting structure. It is essential for calculating the total depth of the bridge deck, considering additional components and traffic. This depth helps determine wind loads acting on the structure.",
        'latex_symbol': r'd',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 1.9,
        'use_std': False,
    },
    {
        'id': 'G5_COMP_8',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(Table8.1)'],
        'title': 'Road restraint system options',
        'description': r"This section lists different types of road restraint systems used on bridges, which include open parapets, solid parapets, and safety barriers. ",
        'latex_symbol': r'restop',
        'type': 'string',
        'unit': '',
        'notation': 'text',
        'default': 'Open parapet or safety barrier',
        'table': 'dropdown',
    },
    {
        'id': 'G5_COMP_9',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(Table8.1)'],
        'title': 'Parapet or barrier placement options',
        'description': r"This section allows the user to select whether the parapet or safety barrier is placed on one side or both sides of the bridge. The placement affects the structural load distribution and wind load calculations.",
        'latex_symbol': r'restpla',
        'type': 'string',
        'unit': '',
        'notation': 'text',
        'default': 'on one side',
        'table': 'dropdown',
    },
    {
        'id': 'G5_COMP_10',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(Table8.1)'],
        'title': 'Total depth for without traffic',
        'description': r"This field allows users to input the total depth of the bridge deck when there is no traffic and safety restraints (such as parapets or barriers) are installed on one side and on both side of the bridge.",
        'latex_symbol': r'd_{tot}',
        'latex_equation': r'd + 0.3',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G5_COMP_8', 'G5_COMP_9', 'G5_COMP_7', 'G5_COMP_11'],
        'table': 'formula',
    },
    {
        'id': 'G5_COMP_11',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(Figure8.5)'],
        'title': 'Height for safety barriers or restraints',
        'description': r"This refers to the height added to the total depth of the bridge deck to account for the presence of safety barriers, parapets, or other restraints. This value is used in wind load calculations when the structure includes such elements, increasing the overall height of the section exposed to wind forces.",
        'latex_symbol': r'd_{1}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 4.24,
        'use_std': False,
    },
    {
        'id': 'G5_COMP_12',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.4(1)'],
        'title': 'Select bridge type',
        'description': r"Users can select the type of bridge for wind load calculations. The two options available are Plated bridges, which refer to bridges with solid beams or girders, and Truss bridges, which are composed of connected elements forming triangular units. Each bridge type requires different wind load considerations due to its structure.",
        'latex_symbol': r'bridgetype',
        'type': 'string',
        'notation': 'text',
        'default': 'Plated bridges',
        'table': 'dropdown',
    },
    {
        'id': 'G5_COMP_13',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.4(1)'],
        'title': 'Wind force acting in the y-direction on bridges',
        'description': r"The wind force in the y-direction is the longitudinal wind load along the span of the bridge, calculated as 25% of the x-direction wind force for plated bridges and 50% for truss bridges.",
        'latex_symbol': r'F_{w,y}',
        'latex_equation': r'0.25 \times F_{w}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G5_COMP_12', 'G5_COMP_1'],
        'table': 'formula',
    },
    {
        'id': 'G5_COMP_14',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.3(1)'],
        'title': 'Force coefficient in the z-direction',
        'description': r"The force coefficient in the z-direction represents the wind forces acting vertically on a bridge deck, both upwards (lift) and downwards. It accounts for factors like deck slope, terrain slope, and wind direction fluctuations. The typical value is ±0.9, depending on whether the wind applies an upward or downward force.",
        'latex_symbol': r'c_{f,z}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'default': 0.9,
        'const': True,
    },
    {
        'id': 'G5_COMP_15',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.3(8.3)'],
        'title': 'Reference area in the z-direction',
        'description': r"The reference area in the z-direction refers to the plan area of the bridge deck exposed to vertical wind forces. It is calculated by multiplying the width of the bridge deck by its length. This area is used to determine the wind load acting vertically (upward or downward) on the deck.",
        'latex_symbol': r'A_{ref,z}',
        'latex_equation': r'A_{ref,z} = b \times L',
        'type': 'number',
        'unit': 'm^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G5_COMP_17', 'G5_COMP_6'],
    },
    {
        'id': 'G5_COMP_16',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.3'],
        'title': 'Wind force acting in the z-direction on bridges',
        'description': r"The wind force in the z-direction represents the vertical force exerted by wind on a bridge deck, either upward (lift) or downward. It is calculated using the air density, basic wind speed, force coefficient in the z-direction, and the reference area in the z-direction.",
        'latex_symbol': r'F_{w,z}',
        'latex_equation': r'F_{w,z} = \frac{1}{2} \times \rho \times v_{b}^{2} \times c_{f,z} \times A_{ref,z} \times 10^{-3}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G3_COMP_25', 'G3_COMP_1', 'G5_COMP_14', 'G5_COMP_15'],
    },
    {
        'id': 'G5_COMP_17',
        'standard': 'EN1991-1-4',
        'reference': ['8.3.1(1)'],
        'title': 'Width of the bridge deck',
        'description': r"The width of the bridge deck refers to the horizontal distance across the deck of the bridge, measured perpendicular to the direction of the span. It is an important dimension used in wind load calculations, as it determines the area exposed to wind forces acting in the x-direction.",
        'latex_symbol': r'b',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 12.95,
        'use_std': False,
    },

    # [EN1992-1-1] Eurocode 2: Design of concrete structures — Part 1-1: General rules and rules for buildings
    {
        'id': 'G6_COMP_1',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(5.14)'],
        'title': 'Slenderness ratio',
        'description': r"The slenderness ratio is a dimensionless value that compares the effective length of a structural member to its radius of gyration. It indicates the likelihood of buckling in compression members, with higher values showing greater susceptibility to buckling.",
        'latex_symbol': r'\lambda',
        'latex_equation': r'\lambda = \frac{l_{0}}{i}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G6_COMP_3', 'G6_COMP_5'],
    },
    {
        'id': "G6_COMP_2",
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(Figure5.7)'],
        'title': 'Buckling modes and boundary conditions',
        'description': r"Buckling modes and boundary conditions describe how structural members behave under load based on their end restraints. Different conditions, such as pinned, fixed, or guided ends, affect the way a member experiences buckling.",
        'figure_file': 'detail_g6_comp_2.png',
        'latex_symbol': r'buckmode',
        'type': 'string',
        'notation': 'text',
        'default': 'Pinned Ends',
        'table': 'dropdown',
    },
    {
        'id': 'G6_COMP_3',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(Figure5.7)'],
        'title': 'Effective length of isolated members',
        'description': r"The effective length of isolated members refers to the length used in buckling analysis to represent the portion of a structural member that is free to buckle under load. This length depends on the boundary conditions, such as pinned, fixed, or free ends. It is essential for predicting when and how isolated members, like girders or columns, will buckle.",
        'latex_symbol': r'l_{0}',
        'latex_equation': r'l_{0} = l',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G6_COMP_2', 'G6_COMP_4'],
        'table': 'formula',
    },
    {
        'id': 'G6_COMP_4',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(3)'],
        'title': 'Clear height of compression member',
        'description': r"The clear height refers to the vertical distance between the end restraints of a compression member. It represents the unsupported length of the member, which is critical in determining its slenderness and buckling behavior.",
        'latex_symbol': r'l',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 14.5,
        'use_std': False,
    },
    {
        'id': 'G6_COMP_5',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(1)'],
        'title': 'Radius of gyration',
        'description': r"The radius of gyration represents how a cross-sectional area is distributed around its centroid and is used to calculate a member's slenderness. It is calculated by dividing the second moment of area by the cross-sectional area and then taking the square root. A larger radius of gyration means the member is more resistant to buckling.",
        'latex_symbol': r'i',
        'latex_equation': r'i = \sqrt{\frac{I}{A}}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G6_COMP_8', 'G6_COMP_7'],
    },
    {
        'id': 'G6_COMP_6',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(1)'],
        'title': 'Cross section type selection',
        'description': r"This option allows the user to choose the type of cross section for which the slenderness ratio will be calculated. You can select either a circular section or a rectangular section, and the appropriate radius of gyration will be calculated accordingly.",
        'latex_symbol': r'cosssect',
        'type': 'string',
        'notation': 'text',
        'default': 'circular',
        'table': 'dropdown',
    },
    {
        'id': 'G6_COMP_7',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(1)'],
        'title': 'Second moment of area of section',
        'description': r"The second moment of area measures the distribution of a concrete section's area about a reference axis, representing its resistance to bending. A larger value means greater resistance to bending.",
        'latex_symbol': r'I',
        'latex_equation': r'I = \frac{(\pi \times (\frac{D}{1000})^{4})}{64}',
        'type': 'number',
        'unit': 'm^4',
        'notation': 'scientific',
        'decimal': 2,
        'required': ['G6_COMP_6', 'G6_COMP_9', 'G6_COMP_10', 'G6_COMP_11'],
        'table': 'formula',
    },
    {
        'id': 'G6_COMP_8',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(1)'],
        'title': 'Cross-sectional area',
        'description': r"The cross-sectional area represents the total area of a member's section that resists applied loads. For circular sections, it depends on the diameter, and for rectangular sections, it depends on the width and height.",
        'latex_symbol': r'A',
        'latex_equation': r'I = \frac{(\pi \times (\frac{D}{1000})^{4})}{64}',
        'type': 'number',
        'unit': 'm^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G6_COMP_6', 'G6_COMP_9', 'G6_COMP_10', 'G6_COMP_11'],
        'table': 'formula',
    },
    {
        'id': 'G6_COMP_9',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(1)'],
        'title': 'Diameter for circular sections',
        'description': r"For circular sections, the diameter refers to the distance across the cross section, passing through its center. It is used to calculate the area of circular sections.",
        'latex_symbol': r'D',
        'type': 'number',
        'unit': 'mm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 550,
        'use_std': False,
    },
    {
        'id': 'G6_COMP_10',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(1)'],
        'title': 'Width for rectangular sections',
        'description': r"For rectangular sections, the width refers to the horizontal dimension of the cross section. It is used along with the height to calculate the area of rectangular sections.",
        'latex_symbol': r'b',
        'type': 'number',
        'unit': 'mm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 250,
        'use_std': False,
    },
    {
        'id': 'G6_COMP_11',
        'standard': 'EN1992-1-1',
        'reference': ['5.8.3.2(1)'],
        'title': 'Height for rectangular sections',
        'description': r"For rectangular sections, the height refers to the vertical dimension of the cross section. It is used in combination with the width to calculate the area of rectangular sections.",
        'latex_symbol': r'h',
        'type': 'number',
        'unit': 'mm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 300,
        'use_std': False,
    },

    # [EN1991-2] Eurocode 1: Actions on structures — Part 2: Traffic loads on bridges
    {
        'id': 'G7_COMP_1',
        'standard': 'EN1991-2',
        'reference': ['4.4.1(4.6)'],
        'title': 'Braking forces on road bridges',
        'description': r"Braking forces on road bridges are longitudinal forces acting at the road surface level, caused by vehicles decelerating. These forces are calculated as a fraction of the maximum vertical loads from Load Model 1 applied to Lane 1. The braking force can be uniformly distributed along the axis of the lane, considering the length of the loaded deck.",
        'latex_symbol': r'Q_{lk}',
        'latex_equation': r'Q_{lk} = \min(\max(0.6 \times \alpha_{Q1} \times (2 \times Q_{1k}) + 0.10 \times \alpha_{q1} \times q_{1k} \times w_{1} \times L, 180), 900)',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G7_COMP_10', 'G7_COMP_16', 'G7_COMP_13', 'G7_COMP_11', 'G7_COMP_5', 'G7_COMP_3'],
    },
    {
        'id': 'G7_COMP_2',
        'standard': 'EN1991-2',
        'reference': ['4.4.1(5)'],
        'title': 'Acceleration forces on road bridges',
        'description': r"Acceleration forces on road bridges are longitudinal forces acting at the road surface level, caused by vehicles accelerating. These forces act in the opposite direction to braking forces and are calculated in the same manner, based on the maximum vertical loads from Load Model 1 on Lane 1. The acceleration force may be distributed along the lane axis.",
        'latex_symbol': r'Q_{ck}',
        'latex_equation': r'Q_{ck} = -Q_{lk}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G7_COMP_1'],
    },
    {
        'id': 'G7_COMP_3',
        'standard': 'EN1991-2',
        'reference': ['4.4.1(2)'],
        'title': 'Length of the loaded deck',
        'description': r"The length of the loaded deck refers to the portion of the bridge deck being considered for load application. It is a key factor in determining how forces, such as braking and acceleration, are distributed along the bridge. This length is used in load calculations to ensure the bridge can handle the forces generated by moving vehicles.",
        'latex_symbol': r'L',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 35,
        'use_std': False,
    },
    {
        'id': 'G7_COMP_4',
        'standard': 'EN1991-2',
        'reference': ['4.2.3(1)'],
        'title': 'Carriageway width for road bridge',
        'description': r"The carriageway width refers to the total width of the road surface between the kerbs or vehicle restraint systems on a road bridge. It excludes the width of any central reservations or the vehicle restraint systems themselves. This measurement is crucial for determining the number of notional lanes on the bridge and is a key factor in road bridge load calculations.",
        'latex_symbol': r'w',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 11,
        'use_std': False,
    },
    {
        'id': 'G7_COMP_5',
        'standard': 'EN1991-2',
        'reference': ['4.2.3(Table4.1)'],
        'title': 'Width of notional lanes on a carriageway',
        'description': r"The notional lane width is the assigned width for each lane on a road used for load distribution calculations in bridge design. It helps ensure even distribution of loads across lanes and is determined by the total width of the carriageway.",
        'latex_symbol': r'w_{1}',
        'latex_equation': r'w_{1} = 3',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G7_COMP_4'],
        'table': 'formula',
    },
    {
        'id': 'G7_COMP_6',
        'standard': 'EN1991-2',
        'reference': ['4.2.3(Table4.1)'],
        'title': 'Number of notional lanes on road bridges',
        'description': r"The number of notional lanes on a road bridge represents the maximum number of lanes that can be assigned to the carriageway width. It is determined by dividing the total width of the carriageway by the standard width of a notional lane. This value is essential for calculating load distribution across the bridge.",
        'latex_symbol': r'n_{1}',
        'latex_equation': r'n_{1} = 1',
        'type': 'number',
        'unit': 'ea',
        'notation': 'standard',
        'decimal': 0,
        'required': ['G7_COMP_4'],
        'table': 'formula',
    },
    {
        'id': 'G7_COMP_7',
        'standard': 'EN1991-2',
        'reference': ['4.4.2(Table4.3)'],
        'title': 'Centrifugal force on road bridges',
        'description': r"Centrifugal force is a transverse force acting radially to the axis of the carriageway at the road surface level. It depends on the horizontal radius of the bridge and the total vertical load from vehicles.",
        'latex_symbol': r'Q_{tk}',
        'type': 'object',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G7_COMP_12', 'G7_COMP_8'],
        'table': 'formula',
    },
    {
        'id': 'G7_COMP_8',
        'standard': 'EN1991-2',
        'reference': ['4.4.2(2)'],
        'title': 'Horizontal radius of the carriageway',
        'description': r"The horizontal radius of the carriageway refers to the radius of the curve along the bridge’s centerline. It determines the magnitude of centrifugal forces, with smaller radii resulting in higher forces. When the radius exceeds 1500 meters, the centrifugal force becomes negligible due to the gentle curve.",
        'latex_symbol': r'r',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 1500,
        'use_std': False,
    },
    {
        'id': 'G7_COMP_9',
        'standard': 'EN1991-2',
        'reference': ['4.4.2(4)'],
        'title': 'Transverse braking force on road bridges with radius',
        'description': r"The transverse braking force is a lateral force that occurs due to skew braking or skidding, particularly on road bridges with a horizontal radius. It is calculated as 25% of the longitudinal braking or acceleration force and acts simultaneously with the longitudinal force, ensuring lateral stability on curved sections of the bridge.",
        'latex_symbol': r'Q_{trk}',
        'latex_equation': r'n_{1} = 1',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G7_COMP_8', 'G7_COMP_1'],
        'table': 'formula',
    },
    {
        'id': 'G7_COMP_10',
        'standard': 'EN1991-2',
        'reference': ['4.3.2(3)'],
        'title': 'Adjustment factor for uniformly distributed load  on lane 1',
        'description': r"The adjustment factor for the uniformly distributed load on lane 1 is applied to modify the distributed load values in Load Model 1. This factor ensures that the distributed load on lane 1 reflects actual traffic conditions or specific design requirements, adjusting the load to better represent the traffic's effect on the bridge.",
        'latex_symbol': r'\alpha_{q1}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G7_COMP_11',
        'standard': 'EN1991-2',
        'reference': ['4.3.2(Table4.2)'],
        'title': 'Uniformly distributed load for notional lane 1',
        'description': r"The uniformly distributed load for notional lane 1 refers to the load per square meter applied across the lane in Load Model 1. This load represents the spread of traffic weight, such as that from congested or slow-moving vehicles, over the lane's surface. The load is distributed only over the areas where it creates unfavorable effects on the bridge.",
        'latex_symbol': r'q_{1k}',
        'type': 'number',
        'nuit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 1,
        'default': 9.0,
        'const': True,
    },
    {
        'id': 'G7_COMP_12',
        'standard': 'EN1991-2',
        'reference': ['4.4.2(2)'],
        'title': 'Total vertical load from tandem systems',
        'description': r"The total vertical load refers to the combined load from all tandem systems in Load Model 1, representing the sum of vertical concentrated loads applied to the bridge. This value indicates the maximum weight of vehicles distributed across notional lanes.",
        'latex_symbol': r'Q_{v}',
        'latex_equation': r'Q_{v} = \alpha_{Q1} \times Q_{1k}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 1,
        'required': ['G7_COMP_6', 'G7_COMP_13', 'G7_COMP_16', 'G7_COMP_14', 'G7_COMP_17', 'G7_COMP_15', 'G7_COMP_18'],
        'table': 'formula',
    },
    {
        'id': 'G7_COMP_13',
        'standard': 'EN1991-2',
        'reference': ['4.3.2(3)'],
        'title': 'Adjustment factor for tandem system on lane 1',
        'description': r"The adjustment factor for the tandem load is applied to modify the concentrated load on notional lane 1 in Load Model 1. This factor adjusts the standard load values to account for variations in traffic conditions or design requirements, ensuring the loads reflect the actual traffic composition and bridge usage.",
        'latex_symbol': r'\alpha_{Q1}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G7_COMP_14',
        'standard': 'EN1991-2',
        'reference': ['4.3.2(3)'],
        'title': 'Adjustment factor for tandem system on lane 2',
        'description': r"The adjustment factor for the tandem load is applied to modify the concentrated load on notional lane 2 in Load Model 1.",
        'latex_symbol': r'\alpha_{Q2}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G7_COMP_15',
        'standard': 'EN1991-2',
        'reference': ['4.3.2(3)'],
        'title': 'Adjustment factor for tandem system on lane 3 and beyond',
        'description': r"The adjustment factor for the tandem load is applied to modify the concentrated load on notional lane 3 and beyond in Load Model 1. This factor adjusts the standard load values to account for variations in traffic conditions or design requirements, ensuring that the loads on the third lane and subsequent lanes accurately reflect the traffic composition and bridge usage.",
        'latex_symbol': r'\alpha_{Q3}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G7_COMP_16',
        'standard': 'EN1991-2',
        'reference': ['4.3.2(Table4.2)'],
        'title': 'Tandem system load for notional lane 1',
        'description': r"The load for notional lane 1 refers to the concentrated load from a tandem axle system applied to the first lane in Load Model 1. This load simulates the vertical forces from heavy vehicles. Each axle has two identical wheels, and the load is distributed equally across them, with a square contact surface for each wheel.",
        'latex_symbol': r'Q_{1k}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 1,
        'default': 300.0,
        'const': True,
    },
    {
        'id': 'G7_COMP_17',
        'standard': 'EN1991-2',
        'reference': ['4.3.2(Table4.2)'],
        'title': 'Tandem system load for notional lane 2',
        'description': r"The load for notional lane 2 refers to the concentrated load from a tandem axle system applied to the first lane in Load Model 1.",
        'latex_symbol': r'Q_{2k}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 1,
        'default': 200.0,
        'const': True,
    },
    {
        'id': 'G7_COMP_18',
        'standard': 'EN1991-2',
        'reference': ['4.3.2(Table4.2)'],
        'title': 'Tandem system load for notional lane 3 and beyond',
        'description': r"The load for notional lane 3 and beyond refers to the concentrated load from a tandem axle system applied to the third and subsequent lanes in Load Model 1. This load simulates the vertical forces from heavy vehicles. Each axle has two identical wheels, and the load is distributed equally across them, with a square contact surface for each wheel.",
        'latex_symbol': r'Q_{3k}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 1,
        'default': 100.0,
        'const': True,
    },

    # [EN1991-2] Dynamic Factors for Railway Bridges with Multiple Spans
    {
        'id': 'G8_COMP_1',
        'standard': 'EN1991-2',
        'reference': ['6.4.5.2(6.4)'],
        'title': 'Dynamic factor for carefully maintained tracks',
        'description': r"The dynamic factor is used to account for the dynamic amplification effects caused by the speed and load of trains passing over a railway bridge. Specifically, this factor applies to well-maintained tracks and calculates the dynamic amplification based on the span length of the bridge.",
        'latex_symbol': r'\Phi_{2}',
        'latex_equation': r'\Phi_{2} = \min(\max(\frac{1.44}{\sqrt{L_{\Phi}} - 0.2} + 0.82, 1.0), 1.67)',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_7'],
    },
    {
        'id': 'G8_COMP_2',
        'standard': 'EN1991-2',
        'reference': ['6.4.5.2(6.5)'],
        'title': 'Dynamic factor for standard maintained tracks',
        'description': r"The dynamic factor is used to account for the dynamic amplification effects caused by the speed and load of trains passing over a railway bridge. This factor applies to tracks maintained at a standard level and calculates the dynamic amplification based on the span length of the bridge.",
        'latex_symbol': r'\Phi_{3}',
        'latex_equation': r'\Phi_{3} = \min(\max(\frac{2.16}{\sqrt{L_{\Phi}} - 0.2} + 0.73, 1.0), 2.0)',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_7'],
    },
    {
        'id': 'G8_COMP_3',
        'standard': 'EN1991-2',
        'reference': ['6.4.5.3(Table6.2)'],
        'title': 'Total span length for multi-span main girder',
        'description': r"The total span length for a multi-span main girder, denoted as Li, is the sum of all individual span lengths in a continuous main girder system. This combined length, from L1 to Ln, is crucial for calculating the mean span length, which is used in dynamic analysis and load distribution for multi-span main girders.",
        'latex_symbol': r'L_{i}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 115,
        'use_std': False,
    },
    {
        'id': 'G8_COMP_4',
        'standard': 'EN1991-2',
        'reference': ['6.4.5.3(Table6.2)'],
        'title': 'Number of spans for continuous girders',
        'description': r"The number of spans refers to how many spans are present in a continuous girder or slab system. It is used to calculate the mean span length, and this value helps determine the determinant length, which must not be less than the longest span.",
        'latex_symbol': r'n',
        'type': 'number',
        'unit': 'ea',
        'notation': 'standard',
        'decimal': 0,
        'limits': {
            'ex_min': 1,
        },
        'default': 5,
        'use_std': False,
    },
    {
        'id': 'G8_COMP_5',
        'standard': 'EN1991-2',
        'reference': ['6.4.5.3(Table6.2)'],
        'title': 'Multiplication factor for determinant length',
        'description': r"The multiplication factor is used to adjust the mean span length to calculate the determinant length in continuous girder systems. This factor depends on the number of spans, increasing as the number of spans increases.",
        'latex_symbol': r'k',
        'latex_equation': r'k = 1.2',
        'type': 'number',
        'notation': 'standard',
        'decimal': 2,
        'required': ['G8_COMP_4'],
        'table': 'formula',
    },
    {
        'id': 'G8_COMP_6',
        'standard': 'EN1991-2',
        'reference': ['6.4.5.3(6.6)'],
        'title': 'Mean span length for continuous girders',
        'description': r"The mean span length is the average length of continuous girders or slabs, calculated by summing all span lengths and dividing by the number of spans, and is used to determine the determinant length.",
        'latex_symbol': r'L_{m}',
        'latex_equation': r'L_{m} = \frac{1}{n} \times L_{i}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_4', 'G8_COMP_3'],
    },
    {
        'id': 'G8_COMP_7',
        'standard': 'EN1991-2',
        'reference': ['6.4.5.3(6.7)'],
        'title': 'Determinant length for dynamic factors',
        'description': r"The mean span length is the average length of continuous girders or slabs, calculated by summing all span lengths and dividing by the number of spans, and is used to determine the determinant length.",
        'latex_symbol': r'L_{\Phi}',
        'latex_equation': r'L_{\Phi} = \min(\max(k \times L_{m}, 4), 100)',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_5', 'G8_COMP_6'],
    },
    {
        'id': 'G8_COMP_8',
        'standard': 'EN1991-2',
        'reference': ['6.4.4(1)'],
        'title': 'Maximum line speed at the site',
        'description': r"The maximum line speed at the site refers to the highest speed allowed for trains on a specific section of the railway track. It is crucial for evaluating dynamic effects on structures like bridges, ensuring they are designed to handle the expected train speeds safely.",
        'latex_symbol': r'V',
        'type': 'number',
        'unit': 'km/h',
        'notation': 'standard',
        'decimal': 1,
        'limits': {
            'ex_min': 0,
        },
        'default': 350,
        'use_std': False,
    },
    {
        'id': 'G8_COMP_9',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(3)P'],
        'title': 'Maximum permitted vehicle speed',
        'description': r"The maximum permitted vehicle speed refers to the highest speed at which a train is allowed to travel over a specific section of track or a bridge. It plays a critical role in dynamic calculations, as higher speeds can lead to greater dynamic impacts on the structure, affecting both the design and safety assessments of the bridge.",
        'latex_symbol': r'\nu',
        'latex_equation': r'\nu = \frac{V}{\frac{3600}{1000}}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_8'],
    },
    {
        'id': 'G8_COMP_10',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.2)'],
        'title': 'Dynamic factor for real trains on carefully maintained track (Upper Limit)',
        'description': r"The dynamic factor for real trains on carefully maintained tracks (upper limit) accounts for reduced dynamic impacts in bridges with higher natural frequencies. It ensures accurate load evaluation by considering lower amplification effects due to better track conditions and faster train speeds on stiffer or shorter span bridges.",
        'latex_symbol': r'(1+\phi)_{2,U}',
        'latex_equation': r'(1+\phi)_{2,U} = 1 + \phi\prime_{U} + 0.5 \times \phi\prime\prime_{U}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_14', 'G8_COMP_16'],
    },
    {
        'id': 'G8_COMP_11',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.2)'],
        'title': 'Dynamic factor for real trains on carefully maintained track (Lower Limit)',
        'description': r"The dynamic factor for real trains on carefully maintained tracks (lower limit) evaluates the reduced dynamic impacts in bridges with lower natural frequencies. It considers slower train speeds and the improved track conditions to ensure accurate load assessment in longer span or more flexible bridges.",
        'latex_symbol': r'(1+\phi)_{2,L}',
        'latex_equation': r'(1+\phi)_{2,L} = 1 + \phi\prime_{L} + 0.5 \times \phi\prime\prime_{L}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_15', 'G8_COMP_17'],
    },
    {
        'id': 'G8_COMP_12',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.1)'],
        'title': 'Dynamic factor for real trains on standard maintenance track (Upper Limit)',
        'description': r"The dynamic factor for real trains on standard maintenance tracks (upper limit) reflects the increased dynamic impacts in bridges with higher natural frequencies. It ensures that the structure can manage the amplified stresses and vibrations caused by faster train speeds on tracks with standard maintenance, particularly in stiffer or shorter span bridges.",
        'latex_symbol': r'(1+\phi)_{3,U}',
        'latex_equation': r'(1+\phi)_{3,U} = 1 + \phi\prime_{U} + \phi\prime\prime_{U}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_14', 'G8_COMP_16'],
    },
    {
        'id': 'G8_COMP_13',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.1)'],
        'title': 'Dynamic factor for real trains on standard maintenance track (Lower Limit)',
        'description': r"The dynamic factor for real trains on standard maintenance tracks (upper limit) reflects the increased dynamic impacts in bridges with higher natural frequencies. It ensures that the structure can manage the amplified stresses and vibrations caused by faster train speeds on tracks with standard maintenance, particularly in stiffer or shorter span bridges.",
        'latex_symbol': r'(1+\phi)_{3,L}',
        'latex_equation': r'(1+\phi)_{3,L} = 1 + \phi\prime_{L} + \phi\prime\prime_{L}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_15', 'G8_COMP_17'],
    },
    {
        'id': 'G8_COMP_14',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.3)', 'AnnexC(C.4)'],
        'title': 'Speed-related dynamic component for upper limit',
        'description': r"The speed-related dynamic component for the upper limit accounts for increased dynamic effects on bridges with higher natural frequencies, typically caused by faster trains. It ensures that the amplified impact of speed on these stiffer or shorter span bridges is accurately evaluated.",
        'latex_symbol': r'\phi\prime_{U}',
        'latex_equation': r'\phi\prime_{U} = \frac{K_{U}}{(1 - K_{U} + K_{U}^{4})}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_18'],
        'table': 'formula',
    },
    {
        'id': 'G8_COMP_15',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.3)', 'AnnexC(C.4)'],
        'title': 'Speed-related dynamic component for lower limit',
        'description': r"The speed-related dynamic component for the lower limit evaluates the dynamic effects on bridges with lower natural frequencies, typically caused by slower trains. This ensures that the structure's response to lower speeds is accurately assessed, especially in longer span or more flexible bridges.",
        'latex_symbol': r'\phi\prime_{L}',
        'latex_equation': r'\phi\prime_{L} = \frac{K_{L}}{(1 - K_{L} + K_{L}^{4})}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_19'],
        'table': 'formula',
    },
    {
        'id': 'G8_COMP_16',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.6)'],
        'title': 'Amplification-related dynamic component for upper limit',
        'description': r"The amplification-related dynamic component for the upper limit accounts for additional dynamic effects in bridges with higher natural frequencies. It reflects the amplified dynamic impacts caused by the interaction between the train and the bridge, particularly in shorter span or stiffer bridges.",
        'latex_symbol': r'\phi\prime\prime_{U}',
        'latex_equation': r'\phi\prime\prime_{U} = \max(\frac{\alpha_{v}}{100} \times (56 \times \exp(-(\frac{L_{\Phi}}{10})^{2}) + 50 \times (\frac{L_{\Phi} \times n_{0,U}}{80} - 1) \times \exp(-(\frac{L_{\Phi}}{20})^{2})),0)',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_20', 'G8_COMP_7', 'G8_COMP_21'],
    },
    {
        'id': 'G8_COMP_17',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.6)'],
        'title': 'Amplification-related dynamic component for lower limit',
        'description': r"The amplification-related dynamic component for the lower limit evaluates the dynamic effects on bridges with lower natural frequencies. It assesses the additional impacts caused by train-bridge interaction, particularly in longer span or more flexible bridges.",
        'latex_symbol': r'\phi\prime\prime_{L}',
        'latex_equation': r'\phi\prime\prime_{L} = \max(\frac{\alpha_{v}}{100} \times (56 \times \exp(-(\frac{L_{\Phi}}{10})^{2}) + 50 \times (\frac{L_{\Phi} \times n_{0,L}}{80} - 1) \times \exp(-(\frac{L_{\Phi}}{20})^{2})),0)',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_20', 'G8_COMP_7', 'G8_COMP_22'],
    },
    {
        'id': 'G8_COMP_18',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.5)'],
        'title': 'Speed-frequency ratio for upper limit condition',
        'description': r"The speed-frequency ratio for the upper limit condition assesses potential dynamic amplification in bridges with higher natural frequencies, ensuring the structure can manage amplified dynamic loads caused by faster train speeds.",
        'latex_symbol': r'K_{U}',
        'latex_equation': r'K_{U} = \frac{\nu}{2\times L_{\Phi} \times n_{0,U}}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_9', 'G8_COMP_7', 'G8_COMP_21'],
    },
    {
        'id': 'G8_COMP_19',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.5)'],
        'title': 'Speed-frequency ratio for lower limit condition',
        'description': r"The speed-frequency ratio for the lower limit condition evaluates dynamic amplification in bridges with lower natural frequencies, ensuring the structure can withstand dynamic effects at slower train speeds.",
        'latex_symbol': r'K_{L}',
        'latex_equation': r'K_{L} = \frac{\nu}{2\times L_{\Phi} \times n_{0,L}}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_9', 'G8_COMP_7', 'G8_COMP_22'],
    },
    {
        'id': 'G8_COMP_20',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.7)'],
        'title': 'Speed-related coefficient for dynamic effects',
        'description': r"The speed-related coefficient is used to adjust the dynamic response of a structure based on the train’s speed. Its role is to account for the increased dynamic impact that higher speeds can have on bridges, ensuring that the structure is properly assessed for the effects of moving loads at different speeds.",
        'latex_symbol': r'\alpha_{v}',
        'latex_equation': r'\alpha_{v} = 1',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_9'],
        'table': 'formula',
    },
    {
        'id': 'G8_COMP_21',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.8)'],
        'title': 'Upper limit of the first natural bending frequency',
        'description': r"The upper limit of the first natural bending frequency represents the maximum frequency at which the bridge vibrates under permanent loads. It is used to evaluate the dynamic response of the bridge to train movements, ensuring that the structure is assessed for higher frequency vibrations that may occur in shorter span bridges or stiffer structures.",
        'latex_symbol': r'n_{0,U}',
        'latex_equation': r'n_{0,U} = 94.76 \times L_{\Phi}^{-0.748}',
        'type': 'number',
        'unit': 'Hz',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_7'],
    },
    {
        'id': 'G8_COMP_22',
        'standard': 'EN1991-2',
        'reference': ['AnnexC(C.9)', 'AnnexC(C.10)'],
        'title': 'Lower limit of the first natural bending frequency',
        'description': r"The lower limit of the first natural bending frequency represents the minimum frequency at which the bridge vibrates under permanent loads. It is essential for evaluating the bridge's dynamic response to slower vibrations, which typically occur in longer span bridges or more flexible structures.",
        'latex_symbol': r'n_{0,L}',
        'latex_equation': r'n_{0,L} = 23.58 \times L_{\Phi}^{-0.592}',
        'type': 'number',
        'unit': 'Hz',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_7'],
        'table': 'formula',
    },

    # [EN1991-2] Guide for Calculating Horizontal Forces in Railway Bridges
    {
        'id': 'G9_COMP_1',
        'standard': 'EN1991-2',
        'reference': ['6.5.1(6.17)'],
        'title': 'Characteristic value of centrifugal force as concentrated load',
        'description': r"The characteristic value of centrifugal force as a concentrated load represents the horizontal force acting at specific points on a bridge when a train moves along a curved track. This concentrated load is applied in the case of Load Model 71.",
        'latex_symbol': r'Q_{tk}',
        'latex_equation': r'Q_{tk} = \frac{(\frac{V}{3.6})^{2}}{(g \times r)}\times (f \times Q_{vk})',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_8', 'G9_COMP_3', 'G9_COMP_4', 'G9_COMP_9', 'G9_COMP_6'],
    },
    {
        'id': 'G9_COMP_2',
        'standard': 'EN1991-2',
        'reference': ['6.5.1(6.18)'],
        'title': 'Characteristic value of centrifugal force as uniformly distributed load',
        'description': r"The characteristic value of centrifugal force as a uniformly distributed load represents the horizontal force distributed along the length of a bridge when a train moves along a curved track. This uniformly distributed load is applied in the case of Load Model 71, SW/0, SW/2, and the unloaded train model.",
        'latex_symbol': r'q_{tk}',
        'latex_equation': r'q_{tk} = \frac{(\frac{V}{3.6})^{2}}{(g \times r)}\times (f \times q_{vk})',
        'type': 'number',
        'unit': 'kN/m',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_8', 'G9_COMP_3', 'G9_COMP_4', 'G9_COMP_9', 'G9_COMP_7'],
    },
    {
        'id': 'G9_COMP_3',
        'standard': 'EN1991-1-5',
        'reference': ['6.5.1(4)'],
        'title': 'Acceleration due to gravity',
        'description': r"The acceleration due to gravity refers to the constant force that pulls objects towards the Earth. This value is used in engineering calculations, including those for railway bridges, to account for the influence of gravity on forces such as centrifugal forces.",
        'latex_symbol': r'g',
        'type': 'number',
        'unit': 'm/s^2',
        'notation': 'standard',
        'decimal': 2,
        'default': 9.81,
        'const': True,
    },
    {
        'id': 'G9_COMP_4',
        'standard': 'EN1991-2',
        'reference': ['6.5.1(4)'],
        'title': 'Radius of curvature for railway bridges',
        'description': r"The radius of curvature for railway bridges refers to the radius of the curve along which a railway track is laid on a bridge. It is a key factor in calculating centrifugal forces, as tighter curves (smaller radii) result in higher centrifugal forces on the bridge.",
        'latex_symbol': r'r',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 1500,
        'use_std': False,
    },
    {
        'id': 'G9_COMP_5',
        'standard': 'EN1991-2',
        'reference': ['6.3.1(1)'],
        'title': 'Selection of railway traffic load models',
        'description': r"This section outlines the selection criteria for railway traffic load models used in bridge analysis. Load Model LM71 (and SW/0 for continuous bridges) is used to represent normal rail traffic, while Load Model SW/2 is for heavy loads. The 'unloaded train' model represents the effects of an unloaded train.",
        'latex_symbol': r'rtmodel',
        'type': 'string',
        'notation': 'text',
        'default': 'Load Model 71',
        'table': 'dropdown',
    },
    {
        'id': 'G9_COMP_6',
        'standard': 'EN1991-2',
        'reference': ['6.3'],
        'title': 'Characteristic values of concentrated loads for railway traffic',
        'description': r"The characteristic values of concentrated loads for railway traffic represent the concentrated vertical loads specific to Load Model 71. These loads are crucial for evaluating the impact of concentrated railway traffic loads on bridge structures and ensuring the bridge design accounts for the intense forces exerted by passing trains.",
        'latex_symbol': r'Q_{vk}',
        'latex_equation': r'Q_{vk} = 1000',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 1,
        'required': ['G9_COMP_5'],
        'table': 'formula',
    },
    {
        'id': 'G9_COMP_7',
        'standard': 'EN1991-2',
        'reference': ['6.3'],
        'title': 'Characteristic values of distributed loads for railway traffic',
        'description': r"The characteristic values of distributed loads for railway traffic represent the uniformly distributed vertical loads in train loads and apply to all load models, including Load Model 71, SW/0, SW/2, and the Unloaded Train. For high-speed load models (HSLM), centrifugal forces are determined using the values from Load Model 71.",
        'latex_symbol': r'q_{vk}',
        'latex_equation': r'q_{vk} = 80',
        'type': 'number',
        'unit': 'kN/m',
        'notation': 'standard',
        'decimal': 1,
        'required': ['G9_COMP_5'],
        'table': 'formula',
    },
    {
        'id': 'G9_COMP_8',
        'standard': 'EN1991-2',
        'reference': ['6.5.1(8)'],
        'title': 'Influence length of the loaded part of curved track',
        'description': r"The influence length refers to the length of the curved track on a railway bridge that is directly affected by the train load. It plays a crucial role in determining the horizontal forces, such as centrifugal forces, acting on the structure. A shorter influence length generally leads to higher localized forces, making it essential for accurately calculating load effects on the bridge.",
        'latex_symbol': r'L_{f}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 115,
        'use_std': False,
    },
    {
        'id': 'G9_COMP_9',
        'standard': 'EN1991-2',
        'reference': ['6.5.1(6.19)'],
        'title': 'Reduction factor for centrifugal forces',
        'description': r"The reduction factor for centrifugal forces adjusts the vertical train loads based on train speed and the influence length of the curved track on the bridge, ultimately modifying the centrifugal force acting on the structure.",
        'latex_symbol': r'f',
        'latex_equation': r'f = f_{1}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G9_COMP_5', 'G9_COMP_10'],
        'table': 'formula',
    },
    {
        'id': 'G9_COMP_10',
        'standard': 'EN1991-2',
        'reference': ['6.5.1(6.19)'],
        'title': 'Initial reduction factor for centrifugal force',
        'description': r"The initial reduction factor for centrifugal force is calculated when the train speed exceeds 120km/h and the influence length is greater than 2.88m. This factor is applied to Load Model 71 and SW/0.",
        'latex_symbol': r'f_{1}',
        'latex_equation': r'f = [1 - \frac{(V-120)}{1000} \times (\frac{814}{V} + 1.75)\times(1 - \sqrt{\frac{2.88}{L_{f}}})]',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G8_COMP_8', 'G9_COMP_8'],
        'table': 'formula',
    },
    {
        'id': 'G9_COMP_11',
        'standard': 'EN1991-2',
        'reference': ['6.5.2(2)P'],
        'title': 'Characteristic value of nosing force',
        'description': r"The characteristic value of the nosing force represents a concentrated horizontal force of 100 kN acting at the top of the rails, perpendicular to the track’s centerline. This force is applied on both straight and curved tracks. While it is not multiplied by the dynamic or reduction factors used for centrifugal forces, it must be multiplied by a factor for classified vertical loads if that factor is 1.0 or greater. It is always combined with vertical traffic loads.",
        'latex_symbol': r'Q_{sk}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 1,
        'default': 100,
        'const': True,
    },
    {
        'id': 'G9_COMP_12',
        'standard': 'EN1991-2',
        'reference': ['6.5.3(1)P'],
        'title': 'Influence length for traction and braking forces',
        'description': r"The influence length for traction and braking forces refers to the portion of the structure over which traction and braking forces are applied as a uniformly distributed load. This length is typically determined based on the geometry of the bridge and the portion of the track that is directly affected by the train's braking and traction efforts.",
        'latex_symbol': r'L_{a,b}',
        'latex_equation': r'L_{a,b} = L_{LM71}',
        'type': 'number',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G9_COMP_5', 'G9_COMP_17'],
        'table': 'formula',
    },
    {
        'id': 'G9_COMP_13',
        'standard': 'EN1991-2',
        'reference': ['6.5.3(6.20)'],
        'title': 'Characteristic value of traction force',
        'description': r"The characteristic value of the traction force represents the uniformly distributed longitudinal force acting along the rails due to traction. This force is distributed over the influence length of the structural element and applies to Load Models 71, SW/0, SW/2, and HSLM. The traction force is calculated as 33 kN per meter of the influence length, with a maximum value of 1000 kN.",
        'latex_symbol': r'Q_{lak}',
        'latex_equation': r'Q_{lak} = \min(33 \times L_{a,b}, 1000)',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G9_COMP_5', 'G9_COMP_12'],
        'table': 'formula',
    },
    {
        'id': 'G9_COMP_14',
        'standard': 'EN1991-2',
        'reference': ['6.5.3(6.21)', '6.5.3(6.22)'],
        'title': 'Characteristic value of braking force',
        'description': r"The characteristic value of the braking force, denoted as the uniformly distributed longitudinal force acting along the rails due to braking. This force is distributed over the influence length of the structural element and applies to Load Models 71, SW/0, SW/2, and HSLM.",
        'latex_symbol': r'Q_{lbk}',
        'latex_equation': r'Q_{lbk} = \min(22 \times L_{a,b}, 6000)',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G9_COMP_5', 'G9_COMP_12', 'G9_COMP_15', 'G9_COMP_16'],
        'table': 'formula',
    },
    {
        'id': 'G9_COMP_15',
        'standard': 'EN1991-2',
        'reference': ['6.3.2(Table6.1)'],
        'title': 'Load application length for Load Model SW/0',
        'description': r"The value of 15 meters represents the load application length of the vehicle model acting as a uniformly distributed load in Load Model SW/0. This parameter defines the length over which the load is applied across the span of a continuous bridge. ",
        'latex_symbol': r'a_{SW/0}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 1,
        'default': 15.0,
        'const': True,
    },
    {
        'id': 'G9_COMP_16',
        'standard': 'EN1991-2',
        'reference': ['6.3.2(Table6.1)'],
        'title': 'Load application length for Load Model SW/2',
        'description': r"The value represents the load application length of the vehicle model acting as a uniformly distributed load in Load Model SW/2. This load model is used to represent heavy rail traffic, and the length is crucial for determining how the load is applied across the span of a bridge.",
        'latex_symbol': r'a_{SW/2}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 1,
        'default': 25.0,
        'const': True,
    },
    {
        'id': 'G9_COMP_17',
        'standard': 'EN1991-2',
        'reference': ['6.5.3(1)P'],
        'title': 'Influence length of the loaded part of curved track',
        'description': r"For Load Model 71, the influence length for traction and braking forces refers specifically to the part of the structure subjected to these forces, applied as a uniformly distributed load. This length is typically defined based on the bridge’s geometry and the portion of the track directly affected by the braking and traction efforts of the train under Load Model 71 conditions.",
        'latex_symbol': r'L_{LM71}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 25.4,
        'use_std': False,
    },

    # [EN1991-1-4] Wind Force Calculation on Rectangular Bridge Piers
    {
        'id': 'G10_COMP_1',
        'standard': 'EN1991-1-4',
        'reference': ['5.3(5.3)'],
        'title': 'Wind force acting on a structure',
        'description': r"The wind force acting on a structure or structural component is calculated using the force coefficient, structural factor, peak velocity pressure at the reference height, and the reference area exposed to the wind. This formula helps determine the total wind load on a structure for safety and design purposes.",
        'latex_symbol': r'F_{w}',
        'latex_equation': r'F_{w} = c_{s}c_{d} \times c_{f} \times q_{p}(z) \times A_{ref}',
        'type': 'number',
        'unit': 'kN',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_2', 'G10_COMP_10', 'G3_COMP_24', 'G10_COMP_7'],
    },
    {
        'id': 'G10_COMP_2',
        'standard': 'EN1991-1-4',
        'reference': ['8.2(1)'],
        'title': 'Structural factor',
        'description': r"The structural factor accounts for the combined effects of non-simultaneous peak wind pressures on a structure and the vibrations caused by wind turbulence. This factor may be divided into a size factor and a dynamic factor, depending on the structure's characteristics, such as height, frequency, and shape.",
        'latex_symbol': r'c_{s}c_{d}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'default': 1.0,
        'const': True,
    },
    {
        'id': 'G10_COMP_3',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(2)'],
        'title': 'Length of the rectangular pier being considered',
        'description': r"The length of the rectangular pier refers to the total vertical height of the pier being analyzed. This measurement is crucial for determining the reference area exposed to wind loads and is used in wind force calculations.",
        'latex_symbol': r'l',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 6.5,
        'use_std': False,
    },
    {
        'id': 'G10_COMP_4',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(Figure7.23)'],
        'title': 'Breadth of rectangular sections',
        'description': r"In rectangular sections, the breadth refers to the shorter dimension of the cross-section that is perpendicular to the wind direction. This dimension is critical in determining the reference area and the wind loads acting on the structure.",
        'latex_symbol': r'b',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 2.5,
        'use_std': False,
    },
    {
        'id': 'G10_COMP_5',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(Figure7.23)'],
        'title': 'Depth of rectangular section',
        'description': r"The depth, denoted as d, refers to the longer dimension of the rectangular section perpendicular to the wind direction. It represents the distance between the front and back faces of the section, playing a crucial role in determining the wind forces acting on the structure. The depth is used to calculate the reference area and influences the pressure distribution on the rectangular section.",
        'latex_symbol': r'd',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 3,
        'use_std': False,
    },
    {
        'id': 'G10_COMP_6',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(Figure7.24)'],
        'title': 'Radius of rounded corners',
        'description': r"The radius of rounded corners refers to the curvature of the edges where two surfaces meet, typically on rectangular or square structural sections. This radius is important in wind load calculations, as rounded corners reduce wind resistance and turbulence compared to sharp edges, leading to a more streamlined flow of wind around the structure.",
        'latex_symbol': r'r',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 0,
        'use_std': False,
    },
    {
        'id': 'G10_COMP_7',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(7.10)'],
        'title': 'Reference area for rectangular bridge piers',
        'description': r"For rectangular sections, the reference area is calculated by multiplying the length and the breadth of the structural element. This area represents the surface exposed to wind forces and is essential for calculating wind loads on rectangular bridge piers.",
        'latex_symbol': r'A_{ref}',
        'latex_equation': r'A_{ref} = l \times b',
        'type': 'number',
        'unit': 'm^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_3', 'G10_COMP_4'],
    },
    {
        'id': 'G10_COMP_8',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(Figure7.23)'],
        'title': 'Ratio of depth to breadth',
        'description': r"The ratio of depth to breadth is a key parameter in determining the base force coefficient for rectangular sections. This ratio represents the proportion of the longer side (depth) to the shorter side (breadth) of the section. It influences the aerodynamic behavior and wind load distribution on the structure, and is essential for calculating the base force coefficient accurately.",
        'latex_symbol': r'{d/b}',
        'latex_equation': r'{d/b} = \frac{d}{b}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_5', 'G10_COMP_4'],
    },
    {
        'id': 'G10_COMP_9',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(Figure7.24)'],
        'title': 'Ratio of corner radius to breadth',
        'description': r"The ratio of the corner radius to breadth is used to account for the effect of rounded corners on the aerodynamic performance of rectangular sections. This ratio influences the reduction factor applied to wind load calculations, as larger corner radii reduce turbulence and drag, leading to a more streamlined flow around the section.",
        'latex_symbol': r'{r/b}',
        'latex_equation': r'{r/b} = \frac{r}{b}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_6', 'G10_COMP_4'],
    },
    {
        'id': 'G10_COMP_10',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(7.9)'],
        'title': 'Force coefficient for rectangular sections',
        'description': r"The force coefficient represents the wind load acting on structural elements with rectangular sections. It is calculated by multiplying the base force coefficient by two factors: the reduction factor for rounded corners and the end-effect factor for elements with free-end flow.",
        'latex_symbol': r'c_{f}',
        'latex_equation': r'c_{f} = c_{f,0} \times \psi_{r} \times \psi_{\lambda,R}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_11', 'G10_COMP_12', 'G10_COMP_13'],
    },
    {
        'id': 'G10_COMP_11',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(Figure7.23)'],
        'title': 'Base force coefficient for rectangular sections',
        'description': r"The base force coefficient represents the wind load on rectangular sections with sharp corners and no free-end flow. It serves as the foundation for calculating the total wind force, and adjustments are made with additional factors like the reduction for rounded corners or end effects.",
        'latex_symbol': r'c_{f,0}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_8'],
        'table': 'interpolation',
    },
    {
        'id': 'G10_COMP_12',
        'standard': 'EN1991-1-4',
        'reference': ['7.6(Figure7.24)'],
        'title': 'Reduction factor for rounded corners',
        'description': r"The reduction factor is applied to square sections with rounded corners to account for aerodynamic effects that reduce wind forces. This factor depends on the Reynolds number, which influences the flow characteristics around the section.",
        'latex_symbol': r'\psi_{r}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_9'],
        'table': 'interpolation',
    },
    {
        'id': 'G10_COMP_13',
        'standard': 'EN1991-1-4',
        'reference': ['7.13(Figure7.36)'],
        'title': 'End-effect factor for elements with free-end flow',
        'description': r"The end-effect factor is applied to structural elements with free-end flow to account for the additional wind effects near the free ends of the element. This factor adjusts the wind force calculations for elements where the wind can flow around the ends, influencing the overall aerodynamic behavior.",
        'latex_symbol': r'\psi_{\lambda,R}',
        'latex_equation': r'\psi_{\lambda,R} = 0.1 \times \log_{10}{\lambda_{R}} + 0.6',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_14'],
        'table': 'formula',
    },
    {
        'id': 'G10_COMP_14',
        'standard': 'EN1991-1-4',
        'reference': ['7.13(Table7.16)'],
        'title': 'Effective slenderness ratio for rectangular sections',
        'description': r"The effective slenderness ratio for rectangular sections is key to determining the end-effect factor, which accounts for wind flow around the free ends. A higher slenderness ratio means the structure is more slender and more affected by wind forces, especially at the ends.",
        'latex_symbol': r'\lambda_{R}',
        'latex_equation': r'Q_{lak} = \min(33 \times L_{a,b}, 1000)',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_3', 'G10_COMP_4'],
        'table': 'formula',
    },
    {
        'id': 'G10_COMP_15',
        'standard': 'EN1991-1-4',
        'reference': ['7.13(7.28)'],
        'title': 'Solidity ratio',
        'description': r"The solidity ratio is the ratio of the projected area of the structural members to the overall envelope area. It indicates how solid or open a structure is, affecting its aerodynamic behavior and wind resistance. A higher ratio means the structure is more solid, leading to greater wind forces. This ratio is important for calculating wind loads on various structural sections.",
        'latex_symbol': r'\phi',
        'latex_equation': r'\phi = \min(2 \times \frac{l}{b}, 70)',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_7', 'G10_COMP_16'],
    },
    {
        'id': 'G10_COMP_16',
        'standard': 'EN1991-1-4',
        'reference': ['7.13(3)'],
        'title': 'Overall envelope area',
        'description': r"The overall envelope area refers to the projected area of a structure when viewed perpendicular to the wind direction. It represents the total surface area that is exposed to wind forces. This area is critical in wind load calculations as it determines the amount of force exerted by the wind on the structure. It is typically calculated based on the dimensions of the structure's length and breadth.",
        'latex_symbol': r'A_{c}',
        'latex_equation': r'A_{c} = l \times b',
        'type': 'number',
        'unit': 'm^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G10_COMP_3', 'G10_COMP_4'],
    },

    # [HK.SDM2013] Peak Velocity Pressure for Wind and Traffic Leading Combinations
    {
        'id': 'G11_COMP_1',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(5)'],
        'title': 'Selection of wind action calculation procedure',
        'description': r"The choice of wind action calculation procedure can be categorized into General, Simplified Procedure, or Full Procedure. The General approach is applied broadly, the Simplified Procedure is used for most highway and railway bridges for straightforward calculations, while the Full Procedure is for structures requiring higher structural reliability against wind loading, applied to bridges that meet specific safety criteria.",
        'latex_symbol': r'windproce',
        'type': 'string',
        'notation': 'text',
        'default': 'General',
        'table': 'dropdown',
    },
    {
        'id': 'G11_COMP_2',
        'standard': 'HK.SDM2013',
        'reference': ['3.4'],
        'title': 'Peak velocity pressure for wind leading combinations',
        'description': r"The peak velocity pressure for wind leading combinations is calculated according to three different procedures: General, Simplified, and Full. Each procedure has its own criteria and methods to determine the wind pressure on structures, ensuring proper application for varying levels of structural reliability and exposure.",
        'latex_symbol': r'q_{p,w}',
        'latex_equation': r'q_{p,w} = q_{p}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_1', 'G11_COMP_6', 'G11_COMP_12', 'G11_COMP_15'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_3',
        'standard': 'HK.SDM2013',
        'reference': ['3.4'],
        'title': 'Peak velocity pressure for traffic leading combinations',
        'description': r"The peak velocity pressure for traffic leading combinations is calculated using different procedures, including General, Simplified, and Full. Each procedure ensures that the wind pressure calculations are appropriate for the design requirements of road and railway bridges, considering traffic conditions and structural reliability.",
        'latex_symbol': r'q_{p,t}',
        'latex_equation': r'q_{p,t} = q\prime',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_1', 'G11_COMP_7', 'G11_COMP_13', 'G11_COMP_16'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_4',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(Table3.6)'],
        'title': 'Selection of primary wind measurement location',
        'description': r"Waglan Island provides wind data that reflects extreme, open-sea wind conditions ideal for conservative design. In contrast, the Hong Kong Observatory offers urban wind data typical of pre-development conditions, suitable for urban-focused structural considerations.",
        'latex_symbol': r'windloca',
        'type': 'string',
        'notation': 'text',
        'default': 'Waglan Island',
        'table': 'dropdown',
    },
    {
        'id': 'G11_COMP_5',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(Table3.6)'],
        'title': 'Return period in years',
        'description': r"The return period is the estimated interval of time between occurrences of a specific event, such as extreme weather or natural phenomena, at a given location. It represents the average frequency with which a particular intensity or magnitude of an event is expected to be equaled or exceeded, aiding in risk assessment and planning.",
        'latex_symbol': r'T',
        'type': 'number',
        'unit': 'years',
        'notation': 'standard',
        'decimal': 0,
        'limits': {
            'in_min': 50,
            'in_max': 200,
        },
        'default': 120,
        'use_std': True,
    },
    {
        'id': 'G11_COMP_6',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(3)'],
        'title': 'Peak velocity pressure in general procedure',
        'description': r"Peak velocity pressure calculated by the general procedure represents the pressure exerted by the maximum peak wind velocity on a structure. This measure is vital for designing safe and resilient buildings and bridges, ensuring that structures can withstand wind forces effectively.",
        'latex_symbol': r'q_{p}',
        'latex_equation': r'q_{p} = (\frac{1}{2}) \times \rho \times 10^{-3} \times v_{d}^{2}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_8', 'G11_COMP_9'],
    },
    {
        'id': 'G11_COMP_7',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(3)'],
        'title': 'Hourly mean velocity pressure in general procedure',
        'description': r"Hourly mean velocity pressure in the general procedure refers to the average pressure exerted by the wind on a structure over an hour. It is used to evaluate wind loads and ensure that structures are designed to withstand consistent wind forces.",
        'latex_symbol': r'q\prime',
        'latex_equation': r'q\prime = (\frac{1}{2}) \times \rho \times 10^{-3} \times v\prime^{2}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_8', 'G11_COMP_10'],
    },
    {
        'id': 'G11_COMP_8',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(3)'],
        'title': 'Air density',
        'description': r"Air density refers to the mass per unit volume of air and is an essential factor in calculating wind pressure on structures.",
        'latex_symbol': r'\rho',
        'type': 'number',
        'unit': 'kg/m^3',
        'notation': 'standard',
        'decimal': 3,
        'default': 1.226,
        'const': True,
    },
    {
        'id': 'G11_COMP_9',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(Table3.6)'],
        'title': 'Maximum peak wind velocity',
        'description': r"Maximum peak wind velocity refers to the highest wind speed experienced at a given location and is used to assess the wind loads on structures. This parameter is critical for determining the design wind pressure and ensuring that structures can withstand extreme wind conditions.",
        'latex_symbol': r'v_{d}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_4', 'G11_COMP_48', 'G11_COMP_49'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_10',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(Table3.6)'],
        'title': 'Maximum hourly mean wind velocity',
        'description': r"The maximum hourly mean wind velocity represents the highest average wind speed measured over one hour at a specific location. It is used to assess wind loads and their potential impact on structural stability.",
        'latex_symbol': r'v\prime',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_4', 'G11_COMP_50', 'G11_COMP_51'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_11',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.2.1(Table3.8)'],
        'title': 'Selection of degree of exposure',
        'description': r"The degree of exposure refers to the extent to which a structure is affected by surrounding conditions such as buildings, topography, or open areas. Selecting the appropriate exposure level is crucial for accurately calculating wind pressures on structures.",
        'figure_file': 'detail_g11_comp_11.png',
        'latex_symbol': r'degexpo',
        'type': 'string',
        'notation': 'text',
        'default': '1',
        'table': 'dropdown',
    },
    {
        'id': 'G11_COMP_12',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.2.1(Table3.8)'],
        'title': 'Peak velocity pressure for wind leading in simplified procedure',
        'description': r"The peak velocity pressure derived from this table is used in the simplified procedure to estimate wind loads for wind leading combinations in structural design. For locations with intermediate exposure, engineers should interpolate values using their judgment between those for sheltered and exposed locations, aided by typical examples and descriptions.",
        'latex_symbol': r'q_{p,s,w}',
        'latex_equation': r'q_{p,s,w} = 2.5',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_11'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_13',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.2.2(1)'],
        'title': 'Peak velocity pressure for traffic leading in simplified procedure',
        'description': r"For road bridges, due to the low probability of significant traffic presence at peak wind velocities over 44 m/s, a peak velocity pressure of 1.2 is applied in traffic leading combinations. This ensures the wind action combination value on the bridge and vehicles does not exceed the set limit.",
        'latex_symbol': r'q_{p,s,t}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'default': 1.2,
        'const': True,
    },
    {
        'id': 'G11_COMP_14',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Selection of orography significance for site',
        'description': r"Orography significance for a site refers to whether the location has terrain features, such as hills or cliffs, that increase wind velocities by more than 5%, necessitating the use of an orography factor in wind calculations. If the average slope of the upwind terrain is less than 3°, orographic effects can be disregarded.",
        'figure_file': 'detail_g11_comp_14.png',
        'latex_symbol': r'orosigsite',
        'type': 'string',
        'notation': 'text',
        'default': 'Non-Significant Orography Site',
        'table': 'dropdown',
    },
    {
        'id': 'G11_COMP_15',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3.1'],
        'title': 'Peak velocity pressure for wind leading in full procedure',
        'description': r"The peak velocity pressure for wind leading in the full procedure is calculated using the basic peak velocity pressure and a climate change multiplying factor. The basic peak velocity pressure is determined based on the height of the bridge and the loaded length under consideration.",
        'latex_symbol': r'q_{p}(z)',
        'latex_equation': r'q_{p}(z) = c_{e}(z) \times q_{pb}(z) \times K_{pc}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_14', 'G11_COMP_23', 'G11_COMP_22', 'G11_COMP_20', 'G11_COMP_17'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_16',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3.1'],
        'title': 'Hourly mean velocity pressure',
        'description': r"The hourly mean velocity pressure represents the average wind pressure exerted on a structure over an hour. It is calculated using the basic hourly mean velocity pressure and a climate change multiplying factor, considering the structure's height and loaded length to accurately assess wind effects.",
        'latex_symbol': r'q\prime(z)',
        'latex_equation': r'q\prime(z) = c_{e}(z) \times q_{b}\prime(z) \times K_{pc}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_14', 'G11_COMP_23', 'G11_COMP_22', 'G11_COMP_21', 'G11_COMP_17'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_17',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3.6'],
        'title': 'Climate change velocity pressure multiplying factor',
        'description': r"The climate change velocity pressure multiplying factor is used in the full procedure to adjust wind pressure calculations for the expected future increase in wind speed due to climate change.",
        'latex_symbol': r'K_{pc}',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 2,
        'default': 1.22,
        'const': True,
    },
    {
        'id': 'G11_COMP_18',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Height above ground level',
        'description': r"The height above ground level refers to the vertical distance from the ground surface to the topmost part of the bridge structure, such as the deck or any point exposed to wind.",
        'latex_symbol': r'z',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 10,
            'in_max': 100,
        },
        'default': 40,
        'use_std': False,
    },
    {
        'id': 'G11_COMP_19',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Horizontal wind loaded lengths',
        'description': r"The horizontal wind loaded length is the section of the bridge exposed to wind, producing the most severe effect. It can refer to the base length of a single adverse area or a combined length of multiple adverse areas in continuous structures.",
        'latex_symbol': r'L',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 20,
            'in_max': 2000,
        },
        'default': 100,
        'use_std': False,
    },
    {
        'id': 'G11_COMP_20',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Basic peak velocity pressure',
        'description': r"Basic peak velocity pressure represents the peak wind pressure acting on a structure at a given height above ground level. It varies depending on the height and loaded length of the structure and reflects the potential wind force that can impact the structure.",
        'latex_symbol': r'q_{pb}(z)',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_19'],
        'table': 'bi-interpolation',
    },
    {
        'id': 'G11_COMP_21',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Basic hourly mean velocity pressure',
        'description': r"Basic hourly mean velocity pressure represents the average wind pressure exerted on a structure over an hour at a specific height above ground level. It varies based on the height and loaded length of the structure, providing an assessment of consistent wind force on the structure.",
        'latex_symbol': r'q_{b}\prime(z)',
        'type': 'number',
        'unit': 'kN/m^2',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18'],
        'table': 'interpolation',
    },
    {
        'id': 'G11_COMP_22',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Exposure-based velocity pressure adjustment factor',
        'description': r"These adjustment factors modify the basic peak velocity pressure and basic hourly mean velocity pressure based on the exposure level of a location. They ensure that wind pressure calculations accurately reflect the influence of different exposure conditions.",
        'latex_symbol': r'c_{e}(z)',
        'latex_equation': r'c_{e}(z) = 0.7',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 1,
        'required': ['G11_COMP_11'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_23',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Topographical factor for significant orography',
        'description': r"The topographical factor for significant orography, denoted as co(z), accounts for the influence of terrain features, such as hills and slopes, on wind pressure. This factor adjusts wind pressure calculations to reflect the amplification or reduction of wind forces due to the shape and elevation of the landscape.",
        'latex_symbol': r'c_{o}(z)',
        'latex_equation': r'c_{o}(z) = 1',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_35', 'G11_COMP_24', 'G11_COMP_28', 'G11_COMP_27'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_24',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'name_Details',
        'description': r"The upwind slope is a measure representing the ratio of the height to the length of a hill, ridge, cliff, or escarpment in the direction of the wind. It affects wind velocity as it approaches these topographical features and is defined in Annex A.3 of BS EN 1991-1-4.",
        'latex_symbol': r'\Phi',
        'latex_equation': r'\Phi =\frac{H}{L_{u}}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_25', 'G11_COMP_26'],
    },
    {
        'id': 'G11_COMP_25',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Height of orographic features',
        'description': r"Height refers to the vertical distance from the base level of flat terrain to the top of an orographic feature, such as a hill, ridge, or cliff. This measurement is essential for calculating the upwind slope and determining its effect on wind velocity as illustrated in Figure A.1 of EN 1991-1-4.",
        'latex_symbol': r'H',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 80,
        'use_std': False,
    },
    {
        'id': 'G11_COMP_26',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Actual length of the upwind slope in the wind direction',
        'description': r"Length refers to the horizontal distance from the start of an incline to the point where it reaches the maximum height of an orographic feature. This distance helps in defining the upwind slope, which influences how wind velocity changes as it passes over the terrain, as shown in Figure A.1 of EN 1991-1-4.",
        'latex_symbol': r'L_{u}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 650,
        'use_std': False,
    },
    {
        'id': 'G11_COMP_27',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.10)'],
        'title': 'Terrain and bridge factor',
        'description': r"The terrain and bridge factor is used in wind pressure calculations to account for the impact of the surrounding terrain and bridge structure at a certain height. For short loaded lengths (less than 50 m) and heights less than 25 m, the height above ground level, z, should be taken as 25 m to ensure consistent and accurate interpolation in calculations.",
        'latex_symbol': r's_{b}(z)',
        'latex_equation': r's_{b}(z) = (1 + (\frac{\max(z, 25)}{10})^{-0.4} \times [1.48 - 0.704 \times \ln(\ln{(t)})]) \times (\frac{\max{(z, 25)}}{10})^{0.19}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_19', 'G11_COMP_29'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_28',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.10)'],
        'title': 'Hourly velocity factor',
        'description': r"The hourly velocity factor is used to represent the variation in wind speed over an hour at a given height. This factor helps in adjusting the wind pressure calculations to reflect the actual wind conditions experienced by the structure, accounting for time-dependent changes in wind velocity.",
        'latex_symbol': r's_{c}(z)',
        'latex_equation': r's_{c}(z) = (\frac{z}{10})^{0.19}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18'],
    },
    {
        'id': 'G11_COMP_29',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.10)'],
        'title': 'Gust duration',
        'description': r"Gust duration refers to the time interval over which a gust of wind acts on a structure. It is an important parameter in wind load analysis as it helps in understanding the impact of short-term wind speed fluctuations on the structural response.",
        'latex_symbol': r't',
        'latex_equation': r't = 3',
        'type': 'number',
        'unit': 'sec',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_19'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_30',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Selection of terrain type for s factor',
        'description': r"The calculation for the s factor differs based on whether the terrain type is classified as cliffs and escarpments or hills and ridges. Selecting the appropriate terrain type is essential for applying the correct method in wind pressure assessments. Refer to Figure A.2 and Figure A.3 of BS EN 1991-1-4 for detailed guidance.",
        'figure_file': 'detail_g11_comp_30.png',
        'latex_symbol': r'terrtype',
        'type': 'string',
        'notation': 'text',
        'default': 'Cliffs and Escarpments',
        'table': 'dropdown',
    },
    {
        'id': 'G11_COMP_31',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Effective length of upwind slope',
        'description': r"The effective length of the upwind slope is a measure used to represent the length of the slope that impacts wind behavior, as defined in Table A.2 of BS EN 1991-1-4. It varies based on whether the slope is shallow or steep and affects wind load calculations on structures.",
        'latex_symbol': r'L_{e}',
        'latex_equation': r'L_{e} = L_{u} \times u',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_26', 'G11_COMP_32'],
    },
    {
        'id': 'G11_COMP_32',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Adjustment factor for effective length',
        'description': r"The adjustment factor for effective length is used to modify the calculation of the effective length of the upwind slope based on the steepness of the slope. According to Table A.2 of BS EN 1991-1-4, when the slope is shallow (between 0.05 and 0.3), the factor is set to 1. For steeper slopes (greater than 0.3), the factor is calculated as the ratio of the slope's steepness to 0.3.",
        'latex_symbol': r'u',
        'latex_equation': r'u = 1',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_24'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_33',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Length of downwind slope for hills and ridges',
        'description': r"The length of the downwind slope is the horizontal distance from the crest of a hill or ridge to the end of the slope in the wind direction. This measurement is crucial for assessing how wind velocity and pressure change as the wind moves beyond the crest and is specifically applied in Figure A.3 of BS EN 1991-1-4 for calculating the s factor for hills and ridges.",
        'latex_symbol': r'L_{d}',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'ex_min': 0,
        },
        'default': 400,
        'use_std': False,
    },
    {
        'id': 'G11_COMP_34',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Horizontal distance from the top of the crest',
        'description': r"The horizontal distance from the top of the crest refers to the measurement from the highest point of a hill or ridge to the structure's location on the ground. This distance is important for assessing how wind behaves as it travels over and beyond the crest. The distance is considered negative (-) to the left of the crest and positive (+) to the right of the crest.",
        'latex_symbol': r'x',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
        },
        'default': -150,
        'use_std': False,
    },
    {
        'id': 'G11_COMP_35',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Orographic location factor',
        'description': r"The orographic location factor, denoted as s, is used to account for the impact of terrain features on wind velocity. It is determined from diagrams such as Figure A.2 or Figure A.3, scaled according to the length of the effective upwind slope, and is specified in Annex A.3 of BS EN 1991-1-4.",
        'latex_symbol': r's',
        'latex_equation': r's = s_{ce}',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_30', 'G11_COMP_36', 'G11_COMP_37'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_36',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Orographic location factor for cliffs and escarpments',
        'description': r"The orographic location factor for cliffs and escarpments, denoted as s_ce, is used to evaluate wind behavior over steep terrain features. This factor is derived from Figure A.2 of BS EN 1991-1-4 and is essential for accurate wind load calculations for structures in such terrains.",
        'latex_symbol': r's_{ce}',
        'latex_equation': r's_{ce} = A \times \exp(B \times \frac{x}{L_{u}})',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_34', 'G11_COMP_18', 'G11_COMP_31', 'G11_COMP_32', 'G11_COMP_38', 'G11_COMP_39', 'G11_COMP_26', 'G11_COMP_40', 'G11_COMP_41', 'G11_COMP_42', 'G11_COMP_43', 'G11_COMP_44', 'G11_COMP_45'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_37',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Orographic location factor for hills and ridges',
        'description': r"The orographic location factor for hills and ridges, denoted as s_hr, is used to assess wind behavior over more gradual terrain features. This factor is derived from Figure A.3 of BS EN 1991-1-4 and plays a crucial role in determining wind loads on structures in such areas.",
        'latex_symbol': r's_{hr}',
        'latex_equation': r's_{hr} = A \times \exp(B \times \frac{x}{L_{u}})',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_34', 'G11_COMP_31', 'G11_COMP_32', 'G11_COMP_18', 'G11_COMP_26', 'G11_COMP_33', 'G11_COMP_38', 'G11_COMP_39', 'G11_COMP_46', 'G11_COMP_47'],
        'table': 'formula',
    },
    {
        'id': 'G11_COMP_38',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient A for upwind section calculation',
        'description': r"Coefficient A is used in the calculation of the s factor for the upwind section of all types of orography.",
        'latex_symbol': r'A',
        'latex_equation': r'A = 0.1552 \times (\frac{z}{L_{e}})^{4} - 0.8575 \times (\frac{z}{L_{e}})^{3} + 1.8133 \times (\frac{z}{L_{e}})^{2} - 1.9115 \times (\frac{z}{L_{e}}) + 0.10124',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_39',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient B for upwind section calculation',
        'description': r"Coefficient B is used alongside Coefficient A for the s factor calculation in the upwind section of all orography.",
        'latex_symbol': r'B',
        'latex_equation': r'B = 0.3542 \times (\frac{z}{L_{e}})^{2} - 1.0577 \times (\frac{z}{L_{e}}) + 2.6456',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_40',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient A1 for downwind section calculation',
        'description': r"Coefficient A1 is used for calculating the s factor in the downwind section for cliffs and escarpments. (Cliffs and Escarpments)",
        'latex_symbol': r'A1',
        'latex_equation': r'A1 = -1.3420 * (\log{(\frac{z}{L_{e}})})^{3} - 0.8222 * (\log{(\frac{z}{L_{e}})})^{2} + 0.4609 * (\log{(\frac{z}{L_{e}})}) - 0.0791',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_41',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient B1 for downwind section calculation',
        'description': r"Coefficient B1 is used in the calculation of the s factor for the downwind section for cliffs and escarpments. (Cliffs and Escarpments)",
        'latex_symbol': r'B1',
        'latex_equation': r'B1 = -1.0196 \times (\log{(\frac{z}{L_{e}})})^{3} - 0.8910 \times (\log{(\frac{z}{L_{e}})})^{2} + 0.5343 \times (\log{(\frac{z}{L_{e}})}) - 0.1156',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_42',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient C1 for downwind section calculation',
        'description': r"Coefficient C1 is applied in calculating the s factor for the downwind section of cliffs and escarpments.(Cliffs and Escarpments)",
        'latex_symbol': r'C1',
        'latex_equation': r'C1 = 0.8030 \times (\log{(\frac{z}{L_{e}})})^{3} + 0.4236 \times (\log{(\frac{z}{L_{e}})})^{2} - 0.5738 \times (\log{(\frac{z}{L_{e}})}) + 0.1606',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_43',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient A2 for downwind section calculation',
        'description': r"Coefficient A2 is specifically used for calculating the s factor in the downwind section for cliffs and escarpments when the vertical position is less than 0.1 relative to the effective length.",
        'latex_symbol': r'A2',
        'latex_equation': r'A2 = -1.3420 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})})^{3} - 0.8222 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})})^{2} + 0.4609 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})}) - 0.0791',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_44',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient B2 for downwind section calculation',
        'description': r"Coefficient B2 is used for calculating the s factor in the downwind section for cliffs and escarpments when the vertical position is less than 0.1 relative to the effective length.",
        'latex_symbol': r'B2',
        'latex_equation': r'B2 = -1.0196 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})})^{3} - 0.8910 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})})^{2} + 0.5343 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})}) - 0.1156',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_45',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient C2 for downwind section calculation',
        'description': r"Coefficient C2 is applied to the calculation of the s factor for the downwind section of cliffs and escarpments when the vertical position is less than 0.1 relative to the effective length.",
        'latex_symbol': r'C2',
        'latex_equation': r'C2 = 0.8030 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})})^{3} + 0.4236 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})})^{2} - 0.5738 \times (\log{(\max{(\frac{z}{L_{e}}, 0.1)})}) + 0.1606',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_46',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient A3 for downwind section calculation',
        'description': r"Coefficient A3 is used in the calculation of the s factor for the downwind section of hills and ridges, accounting for the terrain's influence on wind behavior.",
        'latex_symbol': r'A3',
        'latex_equation': r'A3 = 0.1552 \times (\frac{z}{L_{e}})^{4} - 0.8575 \times (\frac{z}{L_{e}})^{3} + 1.8133 \times (\frac{z}{L_{e}})^{2} - 1.9115 \times (\frac{z}{L_{e}}) + 1.0124',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_47',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.3(Table3.9)'],
        'title': 'Coefficient B3 for downwind section calculation',
        'description': r"Coefficient B3 is applied in the s factor calculation for the downwind section of hills and ridges to refine the evaluation of wind pressure effects on structures.",
        'latex_symbol': r'B3',
        'latex_equation': r'B3 = -0.3056 \times (\frac{z}{L_{e}})^{2} + 1.0212 \times (\frac{z}{L_{e}}) - 1.7637',
        'type': 'number',
        'unit': '',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_18', 'G11_COMP_31'],
    },
    {
        'id': 'G11_COMP_48',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(Table3.6)'],
        'title': 'Maximum Peak Wind Velocity for Waglan Island by Return Period',
        'description': r"This section defines the maximum peak wind velocity specific to Waglan Island, determined based on return periods. It represents the expected wind speeds associated with events occurring within 50 to 200 years.",
        'latex_symbol': r'v_{d,wa}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_5'],
        'table': 'interpolation',
    },
    {
        'id': 'G11_COMP_49',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(Table3.6)'],
        'title': 'Maximum Peak Wind Velocity for Hong Kong Observatory by Return Period',
        'description': r"This section specifies the maximum peak wind velocity recorded at the Hong Kong Observatory, categorized by return periods. It provides wind speed values for events expected within return periods of 50 to 200 years.",
        'latex_symbol': r'v_{d,ob}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_5'],
        'table': 'interpolation',
    },
    {
        'id': 'G11_COMP_50',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(Table3.6)'],
        'title': 'Maximum Hourly Mean Wind Velocity for Waglan Island by Return Period',
        'description': r"This section outlines the maximum hourly mean wind velocity at Waglan Island, calculated based on specific return periods. It presents the expected wind speeds for return periods ranging from 50 to 200 years.",
        'latex_symbol': r'v\prime_{wa}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_5'],
        'table': 'interpolation',
    },
    {
        'id': 'G11_COMP_51',
        'standard': 'HK.SDM2013',
        'reference': ['3.4.1(Table3.6)'],
        'title': 'Maximum Hourly Mean Wind Velocity for Hong Kong Observatory by Return Period',
        'description': r"This section defines the maximum hourly mean wind velocity observed at the Hong Kong Observatory, categorized by return periods. It indicates the expected wind speeds for events with return periods ranging from 50 to 200 years.",
        'latex_symbol': r'v\prime_{ob}',
        'type': 'number',
        'unit': 'm/s',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G11_COMP_5'],
        'table': 'interpolation',
    },

    # [HK.SDM-2013] Uniform Bridge Temperature Range Calculation for Hong Kong
    {
        'id': 'G12_COMP_1',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.17)'],
        'title': 'Structure type selection for uniform bridge temperature',
        'description': r"Selecting the structure type as either normal or minor determines which uniform bridge temperature values apply. Normal structures require standard temperatures adjusted for climate change, while minor structures allow simplified temperature values suited to specific structural elements.",
        'latex_symbol': r'tempstruct',
        'type': 'string',
        'notation': 'text',
        'default': 'Normal',
        'table': 'dropdown',
    },
    {
        'id': 'G12_COMP_2',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.3(Figure3.2)'],
        'title': 'Type of superstructure',
        'description': r"Select the type of superstructure for the bridge. The superstructure type affects various design parameters, including load distribution and temperature adjustments.",
        'figure_file': 'detail_g12_comp_2.png',
        'latex_symbol': r'supertype',
        'type': 'string',
        'notation': 'text',
        'default': 'Steel deck on steel girders',
        'table': 'dropdown',
    },
    {
        'id': 'G12_COMP_3',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.3(Figure3.2)'],
        'title': 'Superstructure group',
        'description': r"The superstructure group is automatically assigned as Type 1, Type 2, or Type 3 based on the selected superstructure. Each group type corresponds to specific design parameters and adjustments.",
        'latex_symbol': r'supergroup',
        'type': 'number',
        'notation': 'text',
        'required': ['G12_COMP_2'],
        'default': 'Type1',
        'table': 'text',
    },
    {
        'id': 'G12_COMP_4',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Deck surfacing type for uniform bridge temperature adjustment',
        'description': r"Select the deck surfacing type for the bridge. This choice affects the uniform bridge temperature adjustments, as different surfacing types and depths impact the thermal properties of the structure. Refer to Table 3.18 for specific adjustment values.",
        'latex_symbol': r'dsurftype',
        'type': 'string',
        'notation': 'text',
        'default': 'Unsurfaced Plain',
        'table': 'dropdown',
    },
    {
        'id': 'G12_COMP_5',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Deck surfacing depth',
        'description': r"Enter the surfacing depth for the bridge deck, including any waterproofing layers. This value is used to adjust the uniform bridge temperature based on the thickness of the surfacing.",
        'latex_symbol': r't_{s}',
        'type': 'number',
        'unit': 'mm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 40,
            'in_max': 200,
        },
        'default': 120,
        'use_std': False,
    },
    {
        'id': 'G12_COMP_6',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(5)'],
        'title': 'Height above mean sea level ',
        'description': r"Enter the height above mean sea level for the structure. This value is used to adjust the uniform bridge temperature values based on elevation, as specified in Table 3.17.",
        'latex_symbol': r'amsl',
        'type': 'number',
        'unit': 'm',
        'notation': 'standard',
        'decimal': 3,
        'limits': {
            'in_min': 0,
        },
        'default': 95.5,
        'use_std': False,
    },
    {
        'id': 'G12_COMP_7',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.17)'],
        'title': 'Minimum uniform bridge temperature',
        'description': r"The minimum uniform bridge temperature is the lowest average temperature that a structure might experience across its entirety. This temperature is used as a reference point for calculating potential contraction effects within the structure due to temperature decreases.",
        'latex_symbol': r'T_{e,min}',
        'latex_equation': r'T_{e,min} = -0.5 \times (\frac{amsl}{100}) + T_{a,min}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_6', 'G12_COMP_9'],
    },
    {
        'id': 'G12_COMP_8',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.17)'],
        'title': 'Maximum uniform bridge temperature',
        'description': r"The maximum uniform bridge temperature is the highest average temperature that a structure might experience across its entirety. This temperature serves as a reference for calculating potential expansion effects within the structure due to temperature increases.",
        'latex_symbol': r'T_{e,max}',
        'latex_equation': r's_{hr} = A \times \exp(B \times \frac{x}{L_{u}})',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_1', 'G12_COMP_3', 'G12_COMP_6', 'G12_COMP_10'],
        'table': 'formula',
    },
    {
        'id': 'G12_COMP_9',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Adjustment to minimum uniform bridge temperature',
        'description': r"This value represents the adjustment to the minimum uniform bridge temperature in degrees Celsius based on the deck surfacing depth provided by the user. Refer to Table 3.18 to calculate the adjustment specific to the entered depth.",
        'latex_symbol': r'T_{a,min}',
        'latex_equation': r'T_{a,min} = 0',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_4', 'G12_COMP_3', 'G12_COMP_11'],
        'table': 'formula',
    },
    {
        'id': 'G12_COMP_10',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Adjustment to maximum uniform bridge temperature',
        'description': r"This value represents the adjustment to the maximum uniform bridge temperature in degrees Celsius based on the deck surfacing depth provided by the user. Refer to Table 3.18 to calculate the adjustment specific to the entered depth.",
        'latex_symbol': r'T_{a,max}',
        'latex_equation': r'T_{a,max} = 4',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_4', 'G12_COMP_3', 'G12_COMP_12'],
        'table': 'formula',
    },
    {
        'id': 'G12_COMP_11',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Additional minimum temperature due to deck surfacing depth',
        'description': r"This value represents the additional temperature applied to the minimum uniform bridge temperature based on the user-entered deck surfacing depth. The adjustment is determined by linear interpolation between the standard depths listed in Table 3.18.",
        'latex_symbol': r'T_{s,min}',
        'latex_equation': r'T_{s,min} = T_{s,min,t1}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_3', 'G12_COMP_18', 'G12_COMP_19', 'G12_COMP_20'],
        'table': 'formula',
    },
    {
        'id': 'G12_COMP_12',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Additional maximum temperature due to deck surfacing depth',
        'description': r"This value represents the additional temperature applied to the minimum uniform bridge temperature based on the user-entered deck surfacing depth. The adjustment is determined by linear interpolation between the standard depths listed in Table 3.18.",
        'latex_symbol': r'T_{s,max}',
        'latex_equation': r'T_{s,max} = T_{s,max,t1}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_3', 'G12_COMP_21', 'G12_COMP_22', 'G12_COMP_23'],
        'table': 'formula',
    },
    {
        'id': 'G12_COMP_13',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(6)'],
        'title': 'Initial bridge temperature for contraction',
        'description': r"The initial bridge temperature for contraction is the reference temperature at the time the structure is restrained upon completion of construction. It serves as a baseline for calculating how much the structure contracts as temperatures decrease toward the minimum uniform bridge temperature component.",
        'latex_symbol': r'T_{0,con}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 0,
        'default': 30,
        'const': True,
    },
    {
        'id': 'G12_COMP_14',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(6)'],
        'title': 'Initial bridge temperature for expansion',
        'description': r"The initial bridge temperature for expansion is the reference temperature at the time the structure is restrained upon completion of construction. It provides a baseline for calculating the expansion of the structure as temperatures increase toward the maximum uniform bridge temperature component.",
        'latex_symbol': r'T_{0,exp}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 0,
        'default': 10,
        'const': True,
    },
    {
        'id': 'G12_COMP_15',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.4'],
        'title': 'Range of the uniform bridge temperature component for contraction',
        'description': r"This is the calculated contraction range for the uniform bridge temperature component, determined from the initial bridge temperature at the time of restraint and the minimum uniform bridge temperature, as specified in EN 1991-1-5 Section 6.1.3.3.",
        'latex_symbol': r'\Delta{T_{N,con}}',
        'latex_equation': r'\Delta{T_{N,con}} = T_{0,con} - T_{e,min}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_13', 'G12_COMP_7'],
    },
    {
        'id': 'G12_COMP_16',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.4'],
        'title': 'Range of the uniform bridge temperature component for expansion',
        'description': r"This is the calculated expansion range for the uniform bridge temperature component, based on the initial bridge temperature at the time of restraint and the maximum uniform bridge temperature, as specified in EN 1991-1-5 Section 6.1.3.3.",
        'latex_symbol': r'\Delta{T_{N,exp}}',
        'latex_equation': r'\Delta{T_{N,exp}} = T_{e,max} - T_{0,exp}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_8', 'G12_COMP_14'],
    },
    {
        'id': 'G12_COMP_17',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.4'],
        'title': 'Overall range of the uniform bridge temperature component',
        'description': r"This is the overall temperature range for the uniform bridge temperature component, calculated as the difference between the maximum and minimum uniform bridge temperatures, as specified in EN 1991-1-5 Section 6.1.3.3.",
        'latex_symbol': r'\Delta{T_{N}}',
        'latex_equation': r'\Delta{T_{N}} = T_{e,max} - T_{e,min}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_8', 'G12_COMP_7'],
    },
    {
        'id': 'G12_COMP_18',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Additional minimum temperature due to deck surfacing depth for Type 1',
        'description': r"This section specifies that no additional temperature adjustment is applied to Type 1 structures, as the adjustment value is always zero regardless of the surfacing depth.",
        'latex_symbol': r'T_{s,min,t1}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'default': 0,
        'const': True,
    },
    {
        'id': 'G12_COMP_19',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Additional minimum temperature due to deck surfacing depth for Type 2',
        'description': r"This section defines temperature adjustments for Type 2 structures based on surfacing depth: -2 at 40, 0 at 100, and 3 at 200. Values for other depths are calculated using linear interpolation.",
        'latex_symbol': r'T_{s,min,t2}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_5'],
        'table': 'interpolation',
    },
    {
        'id': 'G12_COMP_20',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Additional minimum temperature due to deck surfacing depth for Type 3',
        'description': r"This section defines temperature adjustments for Type 3 structures based on surfacing depth: -1 at 40, 0 at 100, and 1 at 200. Values for other depths are calculated using linear interpolation.",
        'latex_symbol': r'T_{s,min,t3}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_5'],
        'table': 'interpolation',
    },
    {
        'id': 'G12_COMP_21',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Additional maximum temperature due to deck surfacing depth for Type 1',
        'description': r"This section specifies that no additional maximum temperature adjustment is applied to Type 1 structures, as the adjustment value is always zero regardless of the surfacing depth.",
        'latex_symbol': r'T_{s,max,t1}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'default': 0,
        'const': True,
    },
    {
        'id': 'G12_COMP_22',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Additional maximum temperature due to deck surfacing depth for Type 2',
        'description': r"This section defines the additional maximum temperature adjustments for Type 2 structures based on surfacing depth: 2 at 40, 0 at 100, and -4 at 200. Values for other depths are calculated using linear interpolation.",
        'latex_symbol': r'T_{s,max,t2}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_5'],
        'table': 'interpolation',
    },
    {
        'id': 'G12_COMP_23',
        'standard': 'HK.SDM2013',
        'reference': ['3.5.2(Table3.18)'],
        'title': 'Additional maximum temperature due to deck surfacing depth for Type 3',
        'description': r"This section defines the additional maximum temperature adjustments for Type 3 structures based on surfacing depth: 1 at 40, 0 at 100, and -2 at 200. Values for other depths are calculated using linear interpolation.",
        'latex_symbol': r'T_{s,max,t3}',
        'type': 'number',
        'unit': '°C',
        'notation': 'standard',
        'decimal': 3,
        'required': ['G12_COMP_5'],
        'table': 'interpolation',
    },
]

data_table = [
    {
        "id": "G1_COMP_1",
        "enum": [
            {
                'label': r'steel box girder',
            },
            {
                'label': r'steel truss or plate girder',
            },
            {
                'label': r'composite deck',
            },
            {
                'label': r'concrete slab',
            },
            {
                'label': r'concrete beam',
            },
            {
                'label': r'concrete box girder',
            }
        ],
    },
    {
        "id": "G1_COMP_2",
        "criteria": [
            [
                r'seldeck = steel box girder',
                r'seldeck = steel truss or plate girder',
                r'seldeck = composite deck',
                r'seldeck = concrete slab',
                r'seldeck = concrete beam',
                r'seldeck = concrete box girder',
            ],
        ],
        "data": [
            'Type1',
            'Type1',
            'Type2',
            'Type3',
            'Type3',
            'Type3',
        ],
    },
    {
        "id": "G1_COMP_14",
        "criteria": [
            [
                r'decktype = Type1',
                r'decktype = Type2',
                r'decktype = Type3',
            ],
        ],
        "data": [
            r'T_{e,min} = T_{min,p} - 3',
            r'T_{e,min} = T_{min,p} + 4',
            r'T_{e,min} = T_{min,p} + 8',
        ],
    },
    {
        "id": "G1_COMP_15",
        "criteria": [
            [
                r'decktype = Type1',
                r'decktype = Type2',
                r'decktype = Type3',
            ],
        ],
        "data": [
            r'T_{e,max} = T_{max,p} - 16',
            r'T_{e,max} = T_{max,p} + 4',
            r'T_{e,max} = T_{max,p} + 2',
        ],
    },
    {
        "id": "G2_COMP_6",
        "criteria": [
            [
                r'buckmode = Pinned Ends',
                r'buckmode = Free - Fixed Ends',
                r'buckmode = Pinned - Fixed Ends',
                r'buckmode = Fixed Ends',
                r'buckmode = Guided - Fixed Ends',
            ],
        ],
        "data": [
            r'H_{i} = N \times e_{i} \times \left( \frac{4}{l} \right)',
            r'H_{i} = N \times e_{i} \times \left( \frac{1}{l} \right)',
            r'H_{i} = N \times e_{i} \times \left( \frac{16}{3 \times l} \right)',
            r'H_{i} = N \times e_{i} \times \left( \frac{8}{l} \right)',
            r'H_{i} = N \times e_{i} \times \left( \frac{2}{l} \right)',
        ],
    },
    {
        "id": "G2_COMP_8",
        "criteria": [
            [
                r'buckmode = Pinned Ends',
                r'buckmode = Free - Fixed Ends',
                r'buckmode = Pinned - Fixed Ends',
                r'buckmode = Fixed Ends',
                r'buckmode = Guided - Fixed Ends'
            ],
        ],
        "data": [
            r'l_{0} = l',
            r'l_{0} = 2 \times l',
            r'l_{0} = 0.7 \times l',
            r'l_{0} = 0.5 \times l',
            r'l_{0} = l',
        ],
    },
    {
        "id": "G3_COMP_14",
        "criteria": [
            [
                r'z <= z_{min}',
                r'z > z_{min}',
            ]
        ],
        "data": [
            r'c_{r}(z) = k_{r} \times \ln \left(\frac {z_{min}}{z_{0}} \right)',
            r'c_{r}(z) = k_{r} \times \ln \left(\frac {z}{z_{0}} \right)',
        ],
    },
    {
        "id": "G3_COMP_15",
        "enum": [
            {
                'label': r'[0] Sea or coastal area exposed to the open sea',
                'description': r'Areas directly exposed to the open sea, such as coastal regions with no protection from land (e.g., oceanfront cliffs or beaches).',
            },
            {
                'label': r'[I] Lakes or flat and horizontal area with negligible vegetation and without obstacles',
                'description': r'Flat, open areas like large lakes or plains with minimal vegetation and no significant obstacles (e.g., flat open fields or dried-up lakebeds).',
            },
            {
                'label': r'[II] Area with low vegetation such as grass and isolated obstacles (trees, buildings) with separations of at least 20 obstacle heights',
                'description': r'Areas with low vegetation like grasslands or farmland, with occasional obstacles such as trees or buildings, separated by at least 20 times their height (e.g., rural fields with scattered farmhouses).',
            },
            {
                'label': r'[III] Area with regular cover of vegetation or buildings or with isolated obstacles with separations of maximum 20 obstacle heights (such as villages, suburban terrain, permanent forest)',
                'description': r'Areas with a regular cover of vegetation or buildings, such as suburban areas or forests, with obstacles like trees or houses spaced up to 20 times their height (e.g., villages or small towns surrounded by forests).',
            },
            {
                'label': r'[IV] Area in which at least 15 % of the surface is covered with buildings and their average height exceeds 15 m',
                'description': r'Urbanized areas where at least 15% of the ground surface is covered by tall buildings, typically over 15 meters in height (e.g., city centers with skyscrapers or densely built industrial zones).',
            },
        ],
    },
    {
        "id": "G3_COMP_16",
        "criteria": [
            [
                r'terrain = [0] Sea or coastal area exposed to the open sea',
                r'terrain = [I] Lakes or flat and horizontal area with negligible vegetation and without obstacles',
                r'terrain = [II] Area with low vegetation such as grass and isolated obstacles (trees, buildings) with separations of at least 20 obstacle heights',
                r'terrain = [III] Area with regular cover of vegetation or buildings or with isolated obstacles with separations of maximum 20 obstacle heights (such as villages, suburban terrain, permanent forest)',
                r'terrain = [IV] Area in which at least 15 % of the surface is covered with buildings and their average height exceeds 15 m',
            ],
        ],
        "data": [
            r'z_{0} = 0.003',
            r'z_{0} = 0.010',
            r'z_{0} = 0.050',
            r'z_{0} = 0.300',
            r'z_{0} = 1.000',
        ],
    },
    {
        "id": "G3_COMP_17",
        "criteria": [
            [
                r'terrain = [0] Sea or coastal area exposed to the open sea',
                r'terrain = [I] Lakes or flat and horizontal area with negligible vegetation and without obstacles',
                r'terrain = [II] Area with low vegetation such as grass and isolated obstacles (trees, buildings) with separations of at least 20 obstacle heights',
                r'terrain = [III] Area with regular cover of vegetation or buildings or with isolated obstacles with separations of maximum 20 obstacle heights (such as villages, suburban terrain, permanent forest)',
                r'terrain = [IV] Area in which at least 15 % of the surface is covered with buildings and their average height exceeds 15 m',
            ],
        ],
        "data": [
            r'z_{min} = 1',
            r'z_{min} = 1',
            r'z_{min} = 2',
            r'z_{min} = 5',
            r'z_{min} = 10',
        ],
    },
    {
        "id": "G3_COMP_23",
        "criteria": [
            [
                r'z < z_{min}',
                r'z >= z_{min}',
            ],
        ],
        "data": [
            r'I_{v}(z) = \frac{k_{l}}{c_{o}(z) \times \ln \left( \frac{z_{min}}{z_{0}} \right)}',
            r'I_{v}(z) = \frac{k_{l}}{c_{o}(z) \times \ln \left( \frac{z}{z_{0}} \right)}',
        ],
    },
    {
        "id": "G4_COMP_1",
        "enum": [
            {
                'label': r'Weight density (kN/m^{3})',
            },
            {
                'label': r'Mass density (kg/m^{3})',
            },
        ],
    },
    {
        "id": "G4_COMP_3",
        "criteria": [
            [
                r'densitytype = Weight density (kN/m^{3})',
                r'densitytype = Mass density (kg/m^{3})',
            ],
        ],
        "data": [
            r'kntokg = 1',
            r'kntokg = 1000/g',
        ],
    },
    {
        "id": "G4_COMP_4",
        "data": [
            {
                'Concrete Type': r'Normal weight concrete',
                'Density': r'24 \times kntokg',
            },
            {
                'Concrete Type': r'Normal weight reinforced concrete',
                'value': r'25 \times kntokg',
            },
            {
                'Concrete Type': r'Normal weight pre-stressed concrete',
                'value': r'25 \times kntokg',
            },
            {
                'Concrete Type': r'(unhardened)Normal weight concrete',
                'value': r'25 \times kntokg',
            },
            {
                'Concrete Type': r'(unhardened)Normal weight reinforced concrete',
                'value': r'26 \times kntokg',
            },
            {
                'Concrete Type': r'(unhardened)Normal weight pre-stressed concrete',
                'value': r'26 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_COMP_5",
        "data": [
            {
                'Pavement Type': r'Gussasphalt and asphaltic concrete',
                'Density': r'24 \times kntokg \text{~} 25 \times kntokg',
            },
            {
                'Pavement Type': r'Mastic asphalt',
                'value': r'18 \times kntokg \text{~} 22 \times kntokg',
            },
            {
                'Pavement Type': r'Hot rolled asphalt',
                'value': r'23 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_COMP_6",
        "data": [
            {
                'Pavement Type': r'concrete protective layer',
                'Density': r'25 \times kntokg',
            },
            {
                'Pavement Type': r'normal ballast (e.g. granite, gneiss, etc.)',
                'value': r'20 \times kntokg',
            },
            {
                'Pavement Type': r'basaltic ballast',
                'value': r'26 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_COMP_7",
        "data": [
            {
                'Infill Material Type': r'sand (dry)',
                'Density': r'15 \times kntokg \text{~} 16 \times kntokg',
            },
            {
                'Infill Material Type': r'ballast, gravel (loose)',
                'value': r'15 \times kntokg \text{~} 16 \times kntokg',
            },
            {
                'Infill Material Type': r'hardcore',
                'value': r'18.5 \times kntokg \text{~} 19.5 \times kntokg',
            },
            {
                'Infill Material Type': r'crushed slag',
                'value': r'13.5 \times kntokg \text{~} 14.5 \times kntokg',
            },
            {
                'Infill Material Type': r'packed stone rubble',
                'value': r'20.5 \times kntokg \text{~} 21.5 \times kntokg',
            },
            {
                'Infill Material Type': r'puddle clay',
                'value': r'18.5 \times kntokg \text{~} 19.5 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_COMP_8",
        "data": [
            {
                'Track Components': r'2 rails UIC 60',
                'Density': r'1.2 \times kntokg',
            },
            {
                'Track Components': r'prestressed concrete sleeper with track fastenings',
                'Density': r'4.8 \times kntokg',
            },
            {
                'Track Components': r'timber sleepers with track fastenings',
                'Density': r'1.9 \times kntokg',
            },
        ],
    },
    {
        "id": "G4_COMP_9",
        "data": [
            {
                'Track Components': r'2 rails UIC 60 with track fastenings',
                'Density': r'1.7 \times kntokg',
            },
            {
                'Track Components': r'bridge beam and guard rails',
                'Density': r'4.9 \times kntokg',
            },
        ],
    },
    {
        "id": "G5_COMP_8",
        "enum": [
            {
                'label': r'Open parapet or safety barrier',
            },
            {
                'label': r'Solid parapet or safety barrier',
            },
            {
                'label': r'Open parapet and safety barrier',
            },
        ],
    },
    {
        "id": "G5_COMP_9",
        "enum": [
            {
                'label': r'on one side',
            },
            {
                'label': r'on both sides',
            },
        ],
    },
    {
        "id": "G5_COMP_10",
        "criteria": [
            [
                r'restop = Open parapet or safety barrier',
                r'restop = Solid parapet or safety barrier',
                r'restop = Open parapet and safety barrier',
            ],
            [
                r'restpla = on one side',
                r'restpla = on both sides',
            ],
        ],
        "data": [
            [
                r'd_{tot} = d + 0.3',
                r'd_{tot} = d + 0.6',
            ],
            [
                r'd_{tot} = d + d_{1}',
                r'd_{tot} = d + 2 \times d_{1}',
            ],
            [
                r'd_{tot} = d + 0.6',
                r'd_{tot} = d + 1.2',
            ],
        ],
    },
    {
        "id": "G5_COMP_12",
        "enum": [
            {
                'label': r'Plated bridges',
            },
            {
                'label': r'Truss bridges',
            },
        ],
    },
    {
        "id": "G5_COMP_13",
        "criteria": [
            [
                r'bridgetype = Plated bridges',
                r'bridgetype = Truss bridges',
            ],
        ],
        "data": [
            r'F_{w,y} = 0.25 \times F_{w}',
            r'F_{w,y} = 0.5 \times F_{w}',
        ],
    },
    {
        "id": "G6_COMP_2",
        "enum": [
            {
                'label': r'Pinned Ends',
            },
            {
                'label': r'Free - Fixed Ends',
            },
            {
                'label': r'Pinned - Fixed Ends',
            },
            {
                'label': r'Fixed Ends',
            },
            {
                'label': r'Guided - Fixed Ends',
            },
        ],
    },
    {
        "id": "G6_COMP_3",
        "criteria": [
            [
                r'buckmode = Pinned Ends',
                r'buckmode = Free - Fixed Ends',
                r'buckmode = Pinned - Fixed Ends',
                r'buckmode = Fixed Ends',
                r'buckmode = Guided - Fixed Ends'
            ],
        ],
        "data": [
            r'l_{0} = l',
            r'l_{0} = 2 \times l',
            r'l_{0} = 0.7 \times l',
            r'l_{0} = 0.5 \times l',
            r'l_{0} = l',
        ],
    },
    {
        "id": "G6_COMP_6",
        "enum": [
            {
                'label': r'circular',
            },
            {
                'label': r'rectangular',
            },
        ],
    },
    {
        "id": "G6_COMP_7",
        "criteria": [
            [
                r'cosssect = circular',
                r'cosssect = rectangular',
            ],
        ],
        "data": [
            r'I = \frac{(\pi \times (\frac{D}{1000})^{4})}{64}',
            r'I = \frac{\frac{b}{1000} \times (\frac{h}{1000})^{3}}{12}',
        ],
    },
    {
        "id": "G6_COMP_8",
        "criteria": [
            [
                r'cosssect = circular',
                r'cosssect = rectangular',
            ],
        ],
        "data": [
            r'A = \frac{\pi \times (\frac{D}{1000})^{2}}{4}',
            r'A = \frac{b}{1000} \times \frac{h}{1000}',
        ],
    },
    {
        "id": "G7_COMP_5",
        "criteria": [
            [
                r'w < 5.4',
                r'5.4 <= w < 6',
                r'6 < w',
            ],
        ],
        "data": [
            r'w_{1} = 3',
            r'w_{1} = \frac{w}{2}',
            r'w_{1} = 3',
        ]
    },
    {
        "id": "G7_COMP_6",
        "criteria": [
            [
                r'w < 5.4',
                r'5.4 <= w < 6',
                r'6 < w',
            ],
        ],
        "data": [
            r'n_{1} = 1',
            r'n_{1} = 2',
            r'n_{1} = \lfloor \frac{w}{3} \rfloor',
        ]
    },
    {
        "id": "G7_COMP_7",
        "criteria": [
            [
                r'r = 0',
                r'r < 200',
                r'200 <= r <= 1500',
                r'r > 1500',
            ],
        ],
        "data": [
            r'Q_{tk} = 0',
            r'Q_{tk} = 0.2 \times Q_{v}',
            r'Q_{tk} = \frac{40 \times Q_{v}}{r}',
            r'Q_{tk} = 0',
        ]
    },
    {
        "id": "G7_COMP_9",
        "criteria": [
            [
                r'r = 0',
                r'0 < r <= 1500',
                r'r > 1500',
            ],
        ],
        "data": [
            r'Q_{trk} = 0',
            r'Q_{trk} = 0.25 \times Q_{lk}',
            r'Q_{trk} = 0',
        ]
    },
    {
        "id": "G7_COMP_12",
        "criteria": [
            [
                r'n_{1} = 1',
                r'n_{1} = 2',
                r'2 < n_{1}',
            ],
        ],
        "data": [
            r'Q_{v} = \alpha_{Q1} \times Q_{1k}',
            r'Q_{v} = \alpha_{Q1} \times Q_{1k} + \alpha_{Q2} \times Q_{2k}',
            r'Q_{v} = \alpha_{Q1} \times Q_{1k} + \alpha_{Q2} \times Q_{2k} + \alpha_{Q3} \times (n_{1} - 2)  \times Q_{3k} \times (n_{1} - 2)',
        ]
    },
    {
        "id": "G8_COMP_5",
        "criteria": [
            [
                r'n = 2',
                r'n = 3',
                r'n = 4',
                r'n >= 5',
            ],
        ],
        "data": [
            r'k = 1.2',
            r'k = 1.3',
            r'k = 1.4',
            r'k = 1.5',
        ]
    },
    {
        "id": "G8_COMP_14",
        "criteria": [
            [
                r'K_{U} < 0.76',
                r'K_{U} >= 0.76',
            ],
        ],
        "data": [
            r'\phi\prime_{U} = \frac{K_{U}}{(1 - K_{U} + K_{U}^{4})}',
            r'\phi\prime_{U} = 1.325',
        ]
    },
    {
        "id": "G8_COMP_15",
        "criteria": [
            [
                r'K_{L} < 0.76',
                r'K_{L} >= 0.76',
            ],
        ],
        "data": [
            r'\phi\prime_{L} = \frac{K_{L}}{(1 - K_{L} + K_{L}^{4})}',
            r'\phi\prime_{L} = 1.325',
        ]
    },
    {
        "id": "G8_COMP_20",
        "criteria": [
            [
                r'\nu > 22',
                r'\nu <= 22',
            ],
        ],
        "data": [
            r'\alpha_{v} = 1',
            r'\alpha_{v} = \frac{\nu}{22}',
        ]
    },
    {
        "id": "G8_COMP_22",
        "criteria": [
            [
                r'20 < L_{\Phi}',
                r'20 >= L_{\Phi}',
            ],
        ],
        "data": [
            r'n_{0,L} = 23.58 \times L_{\Phi}^{-0.592}',
            r'n_{0,L} = \frac{80}{L_{\Phi}}',
        ]
    },
    {
        "id": "G9_COMP_5",
        "enum": [
            {
                'label': r'Load Model 71',
            },
            {
                'label': r'Load Model SW/0',
            },
            {
                'label': r'Load Model SW/2',
            },
            {
                'label': r'Unloaded Train',
            },
        ],
    },
    {
        "id": "G9_COMP_6",
        "criteria": [
            [
                r'rtmodel = Load Model 71',
                r'rtmodel = Load Model SW/0',
                r'rtmodel = Load Model SW/2',
                r'rtmodel = Unloaded Train',
            ],
        ],
        "data": [
            r'Q_{vk} = 1000',
            r'Q_{vk} = 0',
            r'Q_{vk} = 0',
            r'Q_{vk} = 0',
        ]
    },
    {
        "id": "G9_COMP_7",
        "criteria": [
            [
                r'rtmodel = Load Model 71',
                r'rtmodel = Load Model SW/0',
                r'rtmodel = Load Model SW/2',
                r'rtmodel = Unloaded Train',
            ],
        ],
        "data": [
            r'q_{vk} = 80',
            r'q_{vk} = 133',
            r'q_{vk} = 150',
            r'q_{vk} = 10',
        ]
    },
    {
        "id": "G9_COMP_9",
        "criteria": [
            [
                r'rtmodel = Load Model 71',
                r'rtmodel = Load Model SW/0',
                r'rtmodel = Load Model SW/2',
                r'rtmodel = Unloaded Train',
            ],
        ],
        "data": [
            r'f = f_{1}',
            r'f = f_{1}',
            r'f = 1',
            r'f = 1',
        ]
    },
    {
        "id": "G9_COMP_10",
        "criteria": [
            [
                r'V>120 \land L_{f} > 2.88',
                r'V>=120 \lor L_{f} >= 2.88',
            ],
        ],
        "data": [
            r'f_{1} = [1 - \frac{(V-120)}{1000} \times (\frac{814}{V} + 1.75)\times(1 - \sqrt{\frac{2.88}{L_{f}}})]',
            r'f_{1} = 1',
        ]
    },
    {
        "id": "G9_COMP_12",
        "criteria": [
            [
                r'rtmodel = Load Model 71',
                r'rtmodel = Load Model SW/0',
                r'rtmodel = Load Model SW/2',
                r'rtmodel = Unloaded Train',
            ],
        ],
        "data": [
            r'L_{a,b} = L_{LM71}',
            r'L_{a,b} = 35.3',
            r'L_{a,b} = 57.0',
            r'L_{a,b} = 0.0',
        ]
    },
    {
        "id": "G9_COMP_13",
        "criteria": [
            [
                r'rtmodel = Load Model 71',
                r'rtmodel = Load Model SW/0',
                r'rtmodel = Load Model SW/2',
                r'rtmodel = Unloaded Train',
            ],
        ],
        "data": [
            r'Q_{lak} = \min(33 \times L_{a,b} , 1000)',
            r'Q_{lak} = \min(33 \times L_{a,b} , 1000)',
            r'Q_{lak} = \min(33 \times L_{a,b} , 1000)',
            r'Q_{lak} = 0',
        ]
    },
    {
        "id": "G9_COMP_14",
        "criteria": [
            [
                r'rtmodel = Load Model 71',
                r'rtmodel = Load Model SW/0',
                r'rtmodel = Load Model SW/2',
                r'rtmodel = Unloaded Train',
            ],
        ],
        "data": [
            r'Q_{lbk} = \min(22 \times L_{a,b} , 6000)',
            r'Q_{lbk} = \min(22 \times a_{SW/0} , 6000)',
            r'Q_{lbk} = 35 \times a_{SW/2}',
            r'Q_{lbk} = 0',
        ]
    },
    {
        "id": "G10_COMP_11",
        "point": {
            r'{d/b}' : [0.1, 0.2, 0.7, 5.0, 10.0, 50.0],
        },
        "data": [
            2.0,
            2.0,
            2.4,
            1.0,
            0.9,
            0.9,
        ]
    },
    {
        "id": "G10_COMP_12",
        "point": {
            r'{r/b}' : [0.0, 0.2, 0.4],
        },
        "data": [
            1.0,
            0.5,
            0.5,
        ]
    },
    {
        "id": "G10_COMP_13",
        "criteria": [
            [
                r'1 <= \lambda_{R} <= 10',
                r'10 < \lambda_{R} <= 100',
                r'100 < \lambda_{R} <= 200',
            ],
        ],
        "data": [
            r'\psi_{\lambda,R} = 0.1 \times \log_{10}{\lambda_{R}} + 0.6',
            r'\psi_{\lambda,R} = 0.25 \times \log_{10}{\lambda_{R}} + 0.45',
            r'\psi_{\lambda,R} = 0.166 \times \log_{10}{\lambda_{R}} + 0.618',
        ]
    },
    {
        "id": "G10_COMP_14",
        "criteria": [
            [
                r'l < 15',
                r'15 <= l < 50',
                r'l >= 50',
            ],
        ],
        "data": [
            r'\lambda_{R} = \min(2 \times \frac{l}{b}, 70)',
            r'\lambda_{R} = \min(\frac{l}{b} \times (\frac{79}{35} - (0.6 \times \frac{l}{35})), 70)',
            r'\lambda_{R} = \min(1.4 \times \frac{l}{b}, 70)',
        ]
    },
    {
        "id": "G11_COMP_1",
        "enum": [
            {
                'label': r'General',
                'description': r'A general procedure for calculating wind actions, involving a measured location selection and calculations based on the return period.',
            },
            {
                'label': r'Simplified',
                'description': r'Suitable for most highway and railway bridges, following Clause 3.4.2 for simpler requirements for wind action calculations.',
            },
            {
                'label': r'Full',
                'description': r'Required for structures needing enhanced structural reliability against wind loading, following Clause 3.4.3 and applied to bridges meeting specific safety criteria.',
            },
        ],
    },
    {
        "id": "G11_COMP_2",
        "criteria": [
            [
                r'windproce = General',
                r'windproce = Simplified',
                r'windproce = Full',
            ],
        ],
        "data": [
            r'q_{p,w} = q_{p}',
            r'q_{p,w} = q_{p,s,w}',
            r'q_{p,w} = q_{p}(z)',
        ]
    },
    {
        "id": "G11_COMP_3",
        "criteria": [
            [
                r'windproce = General',
                r'windproce = Simplified',
                r'windproce = Full',
            ],
        ],
        "data": [
            r'q_{p,t} = q\prime',
            r'q_{p,t} = q_{p,s,t}',
            r'q_{p,t} = q\prime(z)',
        ]
    },
    {
        "id": "G11_COMP_4",
        "enum": [
            {
                'label': r'Waglan Island',
                'description': r'Exposed site providing data based on long-fetch southeasterly winds over open sea, suitable for conservative design considerations.',
            },
            {
                'label': r'Hong Kong Observatory',
                'description': r'Urban location data representative of typical conditions before urban development, offering insights into an exposed urban area.',
            },
        ],
    },
    {
        "id": "G11_COMP_9",
        "criteria": [
            [
                r'windloca = Waglan Island',
                r'windloca = Hong Kong Observatory',
            ],
        ],
        "data": [
            r'v_{d} = v_{d,wa}',
            r'v_{d} = v_{d,ob}',
        ]
    },
    {
        "id": "G11_COMP_10",
        "criteria": [
            [
                r'windloca = Waglan Island',
                r'windloca = Hong Kong Observatory',
            ],
        ],
        "data": [
            r'v_{d} = v\prime_{wa}',
            r'v_{d} = v\prime_{ob}',
        ]
    },
    {
        "id": "G11_COMP_11",
        "enum": [
            {
                'label': r'1',
                'description': r'[Sheltered Location] Sheltered by surrounding buildings and/or topography (e.g., Kowloon Park Drive Flyover)',
            },
            {
                'label': r'2',
                'description': r'Normal exposure (e.g., Castle Road Flyover)',
            },
            {
                'label': r'3',
                'description': r'Elevated situation; not sheltered by buildings or topography (e.g., Tai Po Road Interchange)',
            },
            {
                'label': r'4',
                'description': r'[Exposed Location] Exposed to north-easterly or south-easterly winds across open sea (e.g., Ap Lei Chau Bridge)',
            },
        ],
    },
    {
        "id": "G11_COMP_12",
        "criteria": [
            [
                r'degexpo = 1',
                r'degexpo = 2',
                r'degexpo = 3',
                r'degexpo = 4',
            ],
        ],
        "data": [
            r'q_{p,s,w} = 2.5',
            r'q_{p,s,w} = 2.8',
            r'q_{p,s,w} = 3.3',
            r'q_{p,s,w} = 3.8',
        ]
    },
    {
        "id": "G11_COMP_14",
        "enum": [
            {
                'label': r'Non-Significant Orography Site',
                'description': r'The site does not have significant terrain features, so the topographical factor is not applied.',
            },
            {
                'label': r'Significant Orography Site',
                'description': r'The site has notable terrain features that require applying the topographical factor to wind pressure calculations.',
            },
        ],
    },
    {
        "id": "G11_COMP_15",
        "criteria": [
            [
                r'orosigsite = Non-Significant Orography Site',
                r'orosigsite = Significant Orography Site',
            ],
        ],
        "data": [
            r'q_{p}(z) = c_{e}(z) \times q_{pb}(z) \times K_{pc}',
            r'q_{p}(z) = c_{o}(z)^{2} \times c_{e}(z) \times q_{pb}(z) \times K_{pc}',
        ]
    },
    {
        "id": "G11_COMP_16",
        "criteria": [
            [
                r'orosigsite = Non-Significant Orography Site',
                r'orosigsite = Significant Orography Site',
            ],
        ],
        "data": [
            r'q\prime(z) = c_{e}(z) \times q_{b}\prime(z) \times K_{pc}',
            r'q\prime(z) = c_{o}(z)^{2} \times c_{e}(z) \times q_{b}\prime(z) \times K_{pc}',
        ]
    },
    {
        "id": "G11_COMP_20",
        "point": {
            r'z' : [10, 15, 20, 30, 40, 50, 60, 80, 100, 150, 200],
            r'L' : [20, 100, 200, 400, 600, 1000, 2000, ],
        },
        "data": [
            [4.2, 2.8, 2.4, 2.1, 2.0, 1.8, 1.6],
            [4.2, 2.8, 2.5, 2.2, 2.0, 1.9 ,1.7],
            [4.2, 2.8, 2.5, 2.2, 2.1, 2.0, 1.8],
            [4.2, 2.9, 2.6, 2.3, 2.3, 2.1, 1.9],
            [4.2, 3.0, 2.7, 2.5, 2.3, 2.2, 2.0],
            [4.2, 3.1, 2.8, 2.5, 2.4, 2.3, 2.1],
            [4.3, 3.1, 2.9, 2.6, 2.5, 2.4, 2.2],
            [4.3, 3.3, 3.0, 2.8, 2.7, 2.5, 2.4],
            [4.4, 3.4, 3.1, 2.9, 2.8, 2.7, 2.5],
            [4.6, 3.6, 3.4, 3.2, 3.1, 3.0, 2.8],
            [4.8, 3.8, 3.6, 3.4, 3.3, 3.2, 3.0],
        ]
    },
    {
        "id": "G11_COMP_21",
        "point": {
            r'z' : [10, 15, 20, 30, 40, 50, 60, 80, 100, 150, 200],
        },
        "data": [
            0.8,
            0.9,
            1.0,
            1.1,
            1.3,
            1.4,
            1.5,
            1.7,
            1.8,
            2.1,
            2.3,
        ]
    },
    {
        "id": "G11_COMP_22",
        "criteria": [
            [
                r'degexpo = 1',
                r'degexpo = 2',
                r'degexpo = 3',
                r'degexpo = 4',
            ],
        ],
        "data": [
            r'c_{e}(z) = 0.7',
            r'c_{e}(z) = 0.8',
            r'c_{e}(z) = 0.9',
            r'c_{e}(z) = 1.0',
        ]
    },
    {
        "id": "G11_COMP_23",
        "criteria": [
            [
                r'\Phi < 0.05',
                r'0.05 <= \Phi < 0.3',
                r'\Phi >= 0.3',
            ],
        ],
        "data": [
            r'c_{o}(z) = 1',
            r'c_{o}(z) = 1 + 2 \times s \times \Phi \times \frac{s_{c}(z)}{s_{b}(z)}',
            r'c_{o}(z) = 1 + 0.6 \times s \times Φ \times s_{c}(z) / s_{b}(z)',
        ]
    },
    {
        "id": "G11_COMP_27",
        "criteria": [
            [
                r'L < 50',
                r'L >= 50',
            ],
        ],
        "data": [
            r's_{b}(z) = (1 + (\frac{\max{(z, 25)}}{10})^{-0.4} \times [1.48 - 0.704 \times \ln(\ln{(t)})]) \times (\frac{\max{(z, 25)}}{10})^{0.19}',
            r's_{b}(z) = (1 + (\frac{z}{10})^{-0.4} \times [1.48 - 0.704\times \ln(\ln{(t)})]) \times (\frac{z}{10})^{0.19}',
        ]
    },
    {
        "id": "G11_COMP_29",
        "criteria": [
            [
                r'L <= 20',
                r'L > 20',
            ],
        ],
        "data": [
            r't = 3',
            r't = 0.375 \times L^{0.69}',
        ]
    },
    {
        "id": "G11_COMP_30",
        "enum": [
            {
                'label': r'Cliffs and Escarpments',
                'description': r'Evaluates the impact of wind over steep and abrupt slopes where wind decreases sharply beyond the crest.',
            },
            {
                'label': r'Hills and Ridges',
                'description': r'Represents terrain with gradual or rolling slopes, where wind gradually decreases after crossing the crest.',
            },
        ],
    },
    {
        "id": "G11_COMP_32",
        "criteria": [
            [
                r'0.05 < \Phi < 0.3',
                r'\Phi >= 0.3',
            ],
        ],
        "data": [
            r'u = 1',
            r'u = \frac{\Phi}{0.3}',
        ]
    },
    {
        "id": "G11_COMP_35",
        "criteria": [
            [
                r'terrtype = Cliffs and Escarpments',
                r'terrtype = Hills and Ridges',
            ],
        ],
        "data": [
            r's = s_{ce}',
            r's = s_{hr}',
        ]
    },
    {
        "id": "G11_COMP_36",
        "criteria": [
            [
                r'-1.5 <= \frac{x}{(\frac{L_{e}}{u})} <= 0 \land 0 <= \frac{z}{L_{e}} <= 2.0',
                r'\frac{x}{L_{e}} = 0',
                r'0 < \frac{x}{L_{e}} < 0.1',
                r'0.1 <= \frac{x}{L_{e}} <= 3.5 \land 0.1 <= \frac{z}{L_{e}} <=2.0',
                r'\frac{x}{(\frac{L_{e}}{u})} < -1.5 \lor \frac{x}{L_{e}} > 3.5 \lor \frac{z}{L_{e}} > 2',
            ],
        ],
        "data": [
            r's_{ce} = A \times \exp(B \times \frac{x}{L_{u}})',
            r's_{ce} = A',
            r's_{ce} = A + (\frac{(A2 - B2 + C2 - A)}{0.1}) \times (\frac{x}{L_{e}})A',
            r's_{ce} = A1 \times (\log{(\frac{x}{L_{e}})})^{2} + B1 \times (\log{(\frac{x}{L_{e}})}) + C1',
            r's_{ce} = 0',
        ]
    },
    {
        "id": "G11_COMP_37",
        "criteria": [
            [
                r'-1.5 <= \frac{x}{(\frac{L_{e}}{u})} <= 0 \land 0 <= \frac{z}{L_{e}} <= 2.0',
                r'0 <= \frac{x}{L_{d}} <2.0 \land 0 <= \frac{z}{L_{e}} <= 2.0',
                r'\frac{x}{(\frac{L_{e}}{u})} < -1.5 \lor \frac{x}{L_{d}} > 2.0 \lor \frac{z}{L_{e}} > 2',
            ],
        ],
        "data": [
            r's_{hr} = A \times \exp(B \times \frac{x}{L_{u}})',
            r's_{hr} = A3 \times \exp(B3 \times \frac{x}{L_{d}})',
            r's_{hr} = 0',
        ]
    },
    {
        "id": "G11_COMP_48",
        "point": {
            r'T' : [50, 100, 200],
        },
        "data": [
            71,
            78,
            85,
        ]
    },
    {
        "id": "G11_COMP_49",
        "point": {
            r'T' : [50, 100, 200],
        },
        "data": [
            68,
            75,
            81,
        ]
    },
    {
        "id": "G11_COMP_50",
        "point": {
            r'T' : [50, 100, 200],
        },
        "data": [
            44,
            48,
            53,
        ]
    },
    {
        "id": "G11_COMP_51",
        "point": {
            r'T' : [50, 100, 200],
        },
        "data": [
            41,
            45,
            50,
        ]
    },
    {
        "id": "G12_COMP_1",
        "enum": [
            {
                'label': r'Normal',
                'description': r'Uses standard uniform temperatures, adjusted for climate change effects.',
            },
            {
                'label': r'Minor',
                'description': r'Uses simplified temperatures for foot/cycle track bridges, carriageway joints, and temporary erection loading.',
            },
        ],
    },
    {
        "id": "G12_COMP_2",
        "enum": [
            {
                'label': r'Steel deck on steel girders',
                'description': r'A steel deck supported by steel girders, commonly used for high load capacity and flexibility in design.',
            },
            {
                'label': r'Steel deck on steel truss or plate girders',
                'description': r'A steel deck supported by steel truss or plate girders, offering enhanced structural integrity and load distribution.',
            },
            {
                'label': r'Concrete deck on steel box',
                'description': r'A concrete deck supported by a steel box girder, providing high torsional rigidity and strength, ideal for longer spans or curved bridges.',
            },
            {
                'label': r'Concrete deck on truss or plate girders',
                'description': r'A concrete deck supported by truss or plate girders, offering enhanced load distribution and stability, suitable for medium to long spans.',
            },
            {
                'label': r'Concrete slab',
                'description': r'A solid concrete slab superstructure, typically for smaller spans and simpler load distribution.',
            },
            {
                'label': r'Concrete beams',
                'description': r'Concrete beams as the primary supporting structure, suitable for moderate to long spans.',
            },
            {
                'label': r'Concrete box girder',
                'description': r'A concrete box girder structure, known for its strength, torsional resistance, and suitability for curved spans.',
            },
        ],
    },
    {
        "id": "G12_COMP_3",
        "criteria": [
            [
                r'supertype = Steel deck on steel girders',
                r'supertype = Steel deck on steel truss or plate girders',
                r'supertype = Concrete deck on steel box',
                r'supertype = Concrete deck on truss or plate girders',
                r'supertype = Concrete slab',
                r'supertype = Concrete beams',
                r'supertype = Concrete box girder',
            ],
        ],
        "data": [
            'Type1',
            'Type1',
            'Type2',
            'Type2',
            'Type3',
            'Type3',
            'Type3',
        ],
    },
    {
        "id": "G12_COMP_4",
        "enum": [
            {
                'label': r'Unsurfaced Plain',
                'description': r'Deck with no surfacing, typically exposed concrete.',
            },
            {
                'label': r'Unsurfaced Trafficked or Waterproofed',
                'description': r'Deck with no surfacing but designed for traffic or waterproofed for additional protection.',
            },
            {
                'label': r'Surfacing Depths',
                'description': r'Deck with a specific surfacing depth (e.g., 40 mm, 100 mm, etc.), including waterproofing layers.',
            },
        ],
    },
    {
        "id": "G12_COMP_8",
        "criteria": [
            [
                r'supergroup = Type1',
                r'supergroup = Type2',
                r'supergroup = Type3',
            ],
            [
                r'tempstruct = Normal',
                r'tempstruct = Minor',
            ],
        ],
        "data": [
            [
                r'T_{e,max} = 55 - (amsl/100) + T_{a,max}',
                r'T_{e,max} = 53 - (amsl/100) + T_{a,max}',
            ],
            [
                r'T_{e,max} = 48 - (amsl/100) + T_{a,max}',
                r'T_{e,max} = 46 - (amsl/100) + T_{a,max}',
            ],
            [
                r'T_{e,max} = 45 - (amsl/100) + T_{a,max}',
                r'T_{e,max} = 43 - (amsl/100) + T_{a,max}',
            ],
        ],
    },
    {
        "id": "G12_COMP_9",
        "criteria": [
            [
                r'dsurftype = Unsurfaced Plain',
                r'dsurftype = Unsurfaced Trafficked or Waterproofed',
                r'dsurftype = Surfacing Depths',
            ],
            [
                r'supergroup = Type1',
                r'supergroup = Type2',
                r'supergroup = Type3',
            ],
        ],
        "data": [
            [
                r'T_{a,min} = 0',
                r'T_{a,min} = -3',
                r'T_{a,min} = -1',
            ],
            [
                r'T_{a,min} = 0',
                r'T_{a,min} = -3',
                r'T_{a,min} = -1',
            ],
            [
                r'T_{a,min} = T_{s,min}',
                r'T_{a,min} = T_{s,min}',
                r'T_{a,min} = T_{s,min}',
            ],
        ],
    },
    {
        "id": "G12_COMP_10",
        "criteria": [
            [
                r'dsurftype = Unsurfaced Plain',
                r'dsurftype = Unsurfaced Trafficked or Waterproofed',
                r'dsurftype = Surfacing Depths',
            ],
            [
                r'supergroup = Type1',
                r'supergroup = Type2',
                r'supergroup = Type3',
            ],
        ],
        "data": [
            [
                r'T_{a,min} = 4',
                r'T_{a,min} = 0',
                r'T_{a,min} = 0',
            ],
            [
                r'T_{a,min} = 2',
                r'T_{a,min} = 4',
                r'T_{a,min} = 2',
            ],
            [
                r'T_{a,min} = T_{s,max}',
                r'T_{a,min} = T_{s,max}',
                r'T_{a,min} = T_{s,max}',
            ],
        ],
    },
    {
        "id": "G12_COMP_11",
        "criteria": [
            [
                r'supergroup = Type1',
                r'supergroup = Type2',
                r'supergroup = Type3',
            ],
        ],
        "data": [
            r'T_{s,min} = T_{s,min,t1}',
            r'T_{s,min} = T_{s,min,t2}',
            r'T_{s,min} = T_{s,min,t3}',
        ],
    },
    {
        "id": "G12_COMP_12",
        "criteria": [
            [
                r'supergroup = Type1',
                r'supergroup = Type2',
                r'supergroup = Type3',
            ],
        ],
        "data": [
            r'T_{s,max} = T_{s,max,t1}',
            r'T_{s,max} = T_{s,max,t2}',
            r'T_{s,max} = T_{s,max,t3}',
        ],
    },
    {
        "id": "G12_COMP_19",
        "point": {
            r't_{s}' : [40, 100, 200],
        },
        "data": [
            -2,
            0,
            3,
        ]
    },
    {
        "id": "G12_COMP_20",
        "point": {
            r't_{s}' : [40, 100, 200],
        },
        "data": [
            -1,
            0,
            1,
        ]
    },
    {
        "id": "G12_COMP_22",
        "point": {
            r't_{s}' : [40, 100, 200],
        },
        "data": [
            -2,
            0,
            -4,
        ]
    },
    {
        "id": "G12_COMP_23",
        "point": {
            r't_{s}' : [40, 100, 200],
        },
        "data": [
            1,
            0,
            -2,
        ]
    },
]